/**--------------------------------------------------------------------------**\
					==========================
					 y_colours - X11 colours!
					==========================
Description:
	This holds the colour information that used to be part of the text system
	but which is far more useful than just for text.  This now supports the full
	set of X11 colours, both by name and by definition.  You can also define
	your own if you so choose (up to 32 - should be enough given that this
	includes the X11 colours).
Legal:
	Version: MPL 1.1
	
	The contents of this file are subject to the Mozilla Public License Version 
	1.1 (the "License"); you may not use this file except in compliance with 
	the License. You may obtain a copy of the License at 
	http://www.mozilla.org/MPL/
	
	Software distributed under the License is distributed on an "AS IS" basis,
	WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
	for the specific language governing rights and limitations under the
	License.
	
	The Original Code is the YSI colours include.
	
	The Initial Developer of the Original Code is Alex "Y_Less" Cole.
	Portions created by the Initial Developer are Copyright (C) 2011
	the Initial Developer. All Rights Reserved.
	
	Contributors:
		ZeeX, koolk, JoeBullet/Google63, g_aSlice/Slice
	
	Thanks:
		JoeBullet/Google63 - Handy arbitrary ASM jump code using SCTRL.
		ZeeX - Very productive conversations.
		koolk - IsPlayerinAreaEx code.
		TheAlpha - Danish translation.
		breadfish - German translation.
		Fireburn - Dutch translation.
		yom - French translation.
		50p - Polish translation.
		Zamaroht - Spanish translation.
		Dracoblue, sintax, mabako, Xtreme, other coders - Producing other modes
			for me to strive to better.
		Pixels^ - Running XScripters where the idea was born.
		Matite - Pestering me to release it and using it.
	
	Very special thanks to:
		Thiadmer - PAWN, whose limits continue to amaze me!
		Kye/Kalcor - SA:MP.
		SA:MP Team past, present and future - SA:MP.
	
Version:
	1.0
Changelog:
	16/03/15:
		Added SA:MP and GTA colours.
		Changed naming scheme for consistency.
	06/01/14:
		Added embeddable versions of all colours.
		Cached all macros for faster compilation.
		Added "GT" macro for "~r~" use in SA:MP text.
		Split gametext colours in to a separate file.
	29/11/10:
		Added the ability to use "X11" prefixes in strings.
		Added colourspace resolution for converting {FF0000} to ~r~.
	25/11/10:
		First version.
\**--------------------------------------------------------------------------**/

// Standard colours
#define @SNOW                    0xFFFAFAAA
#define @GHOSTWHITE              0xF8F8FFAA
#define @GHOST_WHITE             0xF8F8FFAA
#define @WHITESMOKE              0xF5F5F5AA
#define @WHITE_SMOKE             0xF5F5F5AA
#define @GAINSBORO               0xDCDCDCAA
#define @FLORALWHITE             0xFFFAF0AA
#define @FLORAL_WHITE            0xFFFAF0AA
#define @OLDLACE                 0xFDF5E6AA
#define @OLD_LACE                0xFDF5E6AA
#define @LINEN                   0xFAF0E6AA
#define @ANTIQUEWHITE            0xFAEBD7AA
#define @ANTIQUE_WHITE           0xFAEBD7AA
#define @PAPAYAWHIP              0xFFEFD5AA
#define @PAPAYA_WHIP             0xFFEFD5AA
#define @BLANCHEDALMOND          0xFFEBCDAA
#define @BLANCHED_ALMOND         0xFFEBCDAA
#define @BISQUE                  0xFFE4C4AA
#define @PEACHPUFF               0xFFDAB9AA
#define @PEACH_PUFF              0xFFDAB9AA
#define @NAVAJOWHITE             0xFFDEADAA
#define @NAVAJO_WHITE            0xFFDEADAA
#define @MOCCASIN                0xFFE4B5AA
#define @CORNSILK                0xFFF8DCAA
#define @IVORY                   0xFFFFF0AA
#define @LEMONCHIFFON            0xFFFACDAA
#define @LEMON_CHIFFON           0xFFFACDAA
#define @SEASHELL                0xFFF5EEAA
#define @HONEYDEW                0xF0FFF0AA
#define @MINTCREAM               0xF5FFFAAA
#define @MINT_CREAM              0xF5FFFAAA
#define @AZURE                   0xF0FFFFAA
#define @ALICEBLUE               0xF0F8FFAA
#define @ALICE_BLUE              0xF0F8FFAA
#define @LAVENDER                0xE6E6FAAA
#define @LAVENDERBLUSH           0xFFF0F5AA
#define @LAVENDER_BLUSH          0xFFF0F5AA
#define @MISTYROSE               0xFFE4E1AA
#define @MISTY_ROSE              0xFFE4E1AA
#define @WHITE                   0xFFFFFFAA
#define @BLACK                   0x000000AA
#define @DARKSLATEGRAY           0x2F4F4FAA
#define @DARK_SLATE_GRAY         0x2F4F4FAA
#define @DARKSLATEGREY           0x2F4F4FAA
#define @DARK_SLATE_GREY         0x2F4F4FAA
#define @DIMGRAY                 0x696969AA
#define @DIM_GRAY                0x696969AA
#define @DIMGREY                 0x696969AA
#define @DIM_GREY                0x696969AA
#define @SLATEGRAY               0x708090AA
#define @SLATE_GRAY              0x708090AA
#define @SLATEGREY               0x708090AA
#define @SLATE_GREY              0x708090AA
#define @LIGHTSLATEGRAY          0x778899AA
#define @LIGHT_SLATE_GRAY        0x778899AA
#define @LIGHTSLATEGREY          0x778899AA
#define @LIGHT_SLATE_GREY        0x778899AA
#define @GRAY                    0xBEBEBEAA
#define @GREY                    0xBEBEBEAA
#define @LIGHTGREY               0xD3D3D3AA
#define @LIGHT_GREY              0xD3D3D3AA
#define @LIGHTGRAY               0xD3D3D3AA
#define @LIGHT_GRAY              0xD3D3D3AA
#define @MIDNIGHTBLUE            0x191970AA
#define @MIDNIGHT_BLUE           0x191970AA
#define @NAVY                    0x000080AA
#define @NAVYBLUE                0x000080AA
#define @NAVY_BLUE               0x000080AA
#define @CORNFLOWERBLUE          0x6495EDAA
#define @CORNFLOWER_BLUE         0x6495EDAA
#define @DARKSLATEBLUE           0x483D8BAA
#define @DARK_SLATE_BLUE         0x483D8BAA
#define @SLATEBLUE               0x6A5ACDAA
#define @SLATE_BLUE              0x6A5ACDAA
#define @MEDIUMSLATEBLUE         0x7B68EEAA
#define @MEDIUM_SLATE_BLUE       0x7B68EEAA
#define @LIGHTSLATEBLUE          0x8470FFAA
#define @LIGHT_SLATE_BLUE        0x8470FFAA
#define @MEDIUMBLUE              0x0000CDAA
#define @MEDIUM_BLUE             0x0000CDAA
#define @ROYALBLUE               0x4169E1AA
#define @ROYAL_BLUE              0x4169E1AA
#define @BLUE                    0x0000FFAA
#define @DODGERBLUE              0x1E90FFAA
#define @DODGER_BLUE             0x1E90FFAA
#define @DEEPSKYBLUE             0x00BFFFAA
#define @DEEP_SKY_BLUE           0x00BFFFAA
#define @SKYBLUE                 0x87CEEBAA
#define @SKY_BLUE                0x87CEEBAA
#define @LIGHTSKYBLUE            0x87CEFAAA
#define @LIGHT_SKY_BLUE          0x87CEFAAA
#define @STEELBLUE               0x4682B4AA
#define @STEEL_BLUE              0x4682B4AA
#define @LIGHTSTEELBLUE          0xB0C4DEAA
#define @LIGHT_STEEL_BLUE        0xB0C4DEAA
#define @LIGHTBLUE               0xADD8E6AA
#define @LIGHT_BLUE              0xADD8E6AA
#define @POWDERBLUE              0xB0E0E6AA
#define @POWDER_BLUE             0xB0E0E6AA
#define @PALETURQUOISE           0xAFEEEEAA
#define @PALE_TURQUOISE          0xAFEEEEAA
#define @DARKTURQUOISE           0x00CED1AA
#define @DARK_TURQUOISE          0x00CED1AA
#define @MEDIUMTURQUOISE         0x48D1CCAA
#define @MEDIUM_TURQUOISE        0x48D1CCAA
#define @TURQUOISE               0x40E0D0AA
#define @CYAN                    0x00FFFFAA
#define @LIGHTCYAN               0xE0FFFFAA
#define @LIGHT_CYAN              0xE0FFFFAA
#define @CADETBLUE               0x5F9EA0AA
#define @CADET_BLUE              0x5F9EA0AA
#define @MEDIUMAQUAMARINE        0x66CDAAAA
#define @MEDIUM_AQUAMARINE       0x66CDAAAA
#define @AQUAMARINE              0x7FFFD4AA
#define @DARKGREEN               0x006400AA
#define @DARK_GREEN              0x006400AA
#define @DARKOLIVEGREEN          0x556B2FAA
#define @DARK_OLIVE_GREEN        0x556B2FAA
#define @DARKSEAGREEN            0x8FBC8FAA
#define @DARK_SEA_GREEN          0x8FBC8FAA
#define @SEAGREEN                0x2E8B57AA
#define @SEA_GREEN               0x2E8B57AA
#define @MEDIUMSEAGREEN          0x3CB371AA
#define @MEDIUM_SEA_GREEN        0x3CB371AA
#define @LIGHTSEAGREEN           0x20B2AAAA
#define @LIGHT_SEA_GREEN         0x20B2AAAA
#define @PALEGREEN               0x98FB98AA
#define @PALE_GREEN              0x98FB98AA
#define @SPRINGGREEN             0x00FF7FAA
#define @SPRING_GREEN            0x00FF7FAA
#define @LAWNGREEN               0x7CFC00AA
#define @LAWN_GREEN              0x7CFC00AA
#define @GREEN                   0x00FF00AA
#define @CHARTREUSE              0x7FFF00AA
#define @MEDIUMSPRINGGREEN       0x00FA9AAA
#define @MEDIUM_SPRING_GREEN     0x00FA9AAA
#define @GREENYELLOW             0xADFF2FAA
#define @GREEN_YELLOW            0xADFF2FAA
#define @LIMEGREEN               0x32CD32AA
#define @LIME_GREEN              0x32CD32AA
#define @YELLOWGREEN             0x9ACD32AA
#define @YELLOW_GREEN            0x9ACD32AA
#define @FORESTGREEN             0x228B22AA
#define @FOREST_GREEN            0x228B22AA
#define @OLIVEDRAB               0x6B8E23AA
#define @OLIVE_DRAB              0x6B8E23AA
#define @DARKKHAKI               0xBDB76BAA
#define @DARK_KHAKI              0xBDB76BAA
#define @KHAKI                   0xF0E68CAA
#define @PALEGOLDENROD           0xEEE8AAAA
#define @PALE_GOLDENROD          0xEEE8AAAA
#define @LIGHTGOLDENRODYELLOW    0xFAFAD2AA
#define @LIGHT_GOLDENROD_YELLOW  0xFAFAD2AA
#define @LIGHTYELLOW             0xFFFFE0AA
#define @LIGHT_YELLOW            0xFFFFE0AA
#define @YELLOW                  0xFFFF00AA
#define @GOLD                    0xFFD700AA
#define @LIGHTGOLDENROD          0xEEDD82AA
#define @LIGHT_GOLDENROD         0xEEDD82AA
#define @GOLDENROD               0xDAA520AA
#define @DARKGOLDENROD           0xB8860BAA
#define @DARK_GOLDENROD          0xB8860BAA
#define @ROSYBROWN               0xBC8F8FAA
#define @ROSY_BROWN              0xBC8F8FAA
#define @INDIANRED               0xCD5C5CAA
#define @INDIAN_RED              0xCD5C5CAA
#define @SADDLEBROWN             0x8B4513AA
#define @SADDLE_BROWN            0x8B4513AA
#define @SIENNA                  0xA0522DAA
#define @PERU                    0xCD853FAA
#define @BURLYWOOD               0xDEB887AA
#define @BEIGE                   0xF5F5DCAA
#define @WHEAT                   0xF5DEB3AA
#define @SANDYBROWN              0xF4A460AA
#define @SANDY_BROWN             0xF4A460AA
#define @TAN                     0xD2B48CAA
#define @CHOCOLATE               0xD2691EAA
#define @FIREBRICK               0xB22222AA
#define @BROWN                   0xA52A2AAA
#define @DARKSALMON              0xE9967AAA
#define @DARK_SALMON             0xE9967AAA
#define @SALMON                  0xFA8072AA
#define @LIGHTSALMON             0xFFA07AAA
#define @LIGHT_SALMON            0xFFA07AAA
#define @ORANGE                  0xFFA500AA
#define @DARKORANGE              0xFF8C00AA
#define @DARK_ORANGE             0xFF8C00AA
#define @CORAL                   0xFF7F50AA
#define @LIGHTCORAL              0xF08080AA
#define @LIGHT_CORAL             0xF08080AA
#define @TOMATO                  0xFF6347AA
#define @ORANGERED               0xFF4500AA
#define @ORANGE_RED              0xFF4500AA
#define @RED                     0xFF0000AA
#define @HOTPINK                 0xFF69B4AA
#define @HOT_PINK                0xFF69B4AA
#define @DEEPPINK                0xFF1493AA
#define @DEEP_PINK               0xFF1493AA
#define @PINK                    0xFFC0CBAA
#define @LIGHTPINK               0xFFB6C1AA
#define @LIGHT_PINK              0xFFB6C1AA
#define @PALEVIOLETRED           0xDB7093AA
#define @PALE_VIOLET_RED         0xDB7093AA
#define @MAROON                  0xB03060AA
#define @MEDIUMVIOLETRED         0xC71585AA
#define @MEDIUM_VIOLET_RED       0xC71585AA
#define @VIOLETRED               0xD02090AA
#define @VIOLET_RED              0xD02090AA
#define @MAGENTA                 0xFF00FFAA
#define @VIOLET                  0xEE82EEAA
#define @PLUM                    0xDDA0DDAA
#define @ORCHID                  0xDA70D6AA
#define @MEDIUMORCHID            0xBA55D3AA
#define @MEDIUM_ORCHID           0xBA55D3AA
#define @DARKORCHID              0x9932CCAA
#define @DARK_ORCHID             0x9932CCAA
#define @DARKVIOLET              0x9400D3AA
#define @DARK_VIOLET             0x9400D3AA
#define @BLUEVIOLET              0x8A2BE2AA
#define @BLUE_VIOLET             0x8A2BE2AA
#define @PURPLE                  0xA020F0AA
#define @MEDIUMPURPLE            0x9370DBAA
#define @MEDIUM_PURPLE           0x9370DBAA
#define @THISTLE                 0xD8BFD8AA
#define @SNOW1                   0xFFFAFAAA
#define @SNOW_1                  0xFFFAFAAA
#define @SNOW2                   0xEEE9E9AA
#define @SNOW_2                  0xEEE9E9AA
#define @SNOW3                   0xCDC9C9AA
#define @SNOW_3                  0xCDC9C9AA
#define @SNOW4                   0x8B8989AA
#define @SNOW_4                  0x8B8989AA
#define @SEASHELL1               0xFFF5EEAA
#define @SEASHELL_1              0xFFF5EEAA
#define @SEASHELL2               0xEEE5DEAA
#define @SEASHELL_2              0xEEE5DEAA
#define @SEASHELL3               0xCDC5BFAA
#define @SEASHELL_3              0xCDC5BFAA
#define @SEASHELL4               0x8B8682AA
#define @SEASHELL_4              0x8B8682AA
#define @ANTIQUEWHITE1           0xFFEFDBAA
#define @ANTIQUE_WHITE_1         0xFFEFDBAA
#define @ANTIQUEWHITE2           0xEEDFCCAA
#define @ANTIQUE_WHITE_2         0xEEDFCCAA
#define @ANTIQUEWHITE3           0xCDC0B0AA
#define @ANTIQUE_WHITE_3         0xCDC0B0AA
#define @ANTIQUEWHITE4           0x8B8378AA
#define @ANTIQUE_WHITE_4         0x8B8378AA
#define @BISQUE1                 0xFFE4C4AA
#define @BISQUE_1                0xFFE4C4AA
#define @BISQUE2                 0xEED5B7AA
#define @BISQUE_2                0xEED5B7AA
#define @BISQUE3                 0xCDB79EAA
#define @BISQUE_3                0xCDB79EAA
#define @BISQUE4                 0x8B7D6BAA
#define @BISQUE_4                0x8B7D6BAA
#define @PEACHPUFF1              0xFFDAB9AA
#define @PEACH_PUFF_1            0xFFDAB9AA
#define @PEACHPUFF2              0xEECBADAA
#define @PEACH_PUFF_2            0xEECBADAA
#define @PEACHPUFF3              0xCDAF95AA
#define @PEACH_PUFF_3            0xCDAF95AA
#define @PEACHPUFF4              0x8B7765AA
#define @PEACH_PUFF_4            0x8B7765AA
#define @NAVAJOWHITE1            0xFFDEADAA
#define @NAVAJO_WHITE_1          0xFFDEADAA
#define @NAVAJOWHITE2            0xEECFA1AA
#define @NAVAJO_WHITE_2          0xEECFA1AA
#define @NAVAJOWHITE3            0xCDB38BAA
#define @NAVAJO_WHITE_3          0xCDB38BAA
#define @NAVAJOWHITE4            0x8B795EAA
#define @NAVAJO_WHITE_4          0x8B795EAA
#define @LEMONCHIFFON1           0xFFFACDAA
#define @LEMON_CHIFFON_1         0xFFFACDAA
#define @LEMONCHIFFON2           0xEEE9BFAA
#define @LEMON_CHIFFON_2         0xEEE9BFAA
#define @LEMONCHIFFON3           0xCDC9A5AA
#define @LEMON_CHIFFON_3         0xCDC9A5AA
#define @LEMONCHIFFON4           0x8B8970AA
#define @LEMON_CHIFFON_4         0x8B8970AA
#define @CORNSILK1               0xFFF8DCAA
#define @CORNSILK_1              0xFFF8DCAA
#define @CORNSILK2               0xEEE8CDAA
#define @CORNSILK_2              0xEEE8CDAA
#define @CORNSILK3               0xCDC8B1AA
#define @CORNSILK_3              0xCDC8B1AA
#define @CORNSILK4               0x8B8878AA
#define @CORNSILK_4              0x8B8878AA
#define @IVORY1                  0xFFFFF0AA
#define @IVORY_1                 0xFFFFF0AA
#define @IVORY2                  0xEEEEE0AA
#define @IVORY_2                 0xEEEEE0AA
#define @IVORY3                  0xCDCDC1AA
#define @IVORY_3                 0xCDCDC1AA
#define @IVORY4                  0x8B8B83AA
#define @IVORY_4                 0x8B8B83AA
#define @HONEYDEW1               0xF0FFF0AA
#define @HONEYDEW_1              0xF0FFF0AA
#define @HONEYDEW2               0xE0EEE0AA
#define @HONEYDEW_2              0xE0EEE0AA
#define @HONEYDEW3               0xC1CDC1AA
#define @HONEYDEW_3              0xC1CDC1AA
#define @HONEYDEW4               0x838B83AA
#define @HONEYDEW_4              0x838B83AA
#define @LAVENDERBLUSH1          0xFFF0F5AA
#define @LAVENDER_BLUSH_1        0xFFF0F5AA
#define @LAVENDERBLUSH2          0xEEE0E5AA
#define @LAVENDER_BLUSH_2        0xEEE0E5AA
#define @LAVENDERBLUSH3          0xCDC1C5AA
#define @LAVENDER_BLUSH_3        0xCDC1C5AA
#define @LAVENDERBLUSH4          0x8B8386AA
#define @LAVENDER_BLUSH_4        0x8B8386AA
#define @MISTYROSE1              0xFFE4E1AA
#define @MISTY_ROSE_1            0xFFE4E1AA
#define @MISTYROSE2              0xEED5D2AA
#define @MISTY_ROSE_2            0xEED5D2AA
#define @MISTYROSE3              0xCDB7B5AA
#define @MISTY_ROSE_3            0xCDB7B5AA
#define @MISTYROSE4              0x8B7D7BAA
#define @MISTY_ROSE_4            0x8B7D7BAA
#define @AZURE1                  0xF0FFFFAA
#define @AZURE_1                 0xF0FFFFAA
#define @AZURE2                  0xE0EEEEAA
#define @AZURE_2                 0xE0EEEEAA
#define @AZURE3                  0xC1CDCDAA
#define @AZURE_3                 0xC1CDCDAA
#define @AZURE4                  0x838B8BAA
#define @AZURE_4                 0x838B8BAA
#define @SLATEBLUE1              0x836FFFAA
#define @SLATE_BLUE_1            0x836FFFAA
#define @SLATEBLUE2              0x7A67EEAA
#define @SLATE_BLUE_2            0x7A67EEAA
#define @SLATEBLUE3              0x6959CDAA
#define @SLATE_BLUE_3            0x6959CDAA
#define @SLATEBLUE4              0x473C8BAA
#define @SLATE_BLUE_4            0x473C8BAA
#define @ROYALBLUE1              0x4876FFAA
#define @ROYAL_BLUE_1            0x4876FFAA
#define @ROYALBLUE2              0x436EEEAA
#define @ROYAL_BLUE_2            0x436EEEAA
#define @ROYALBLUE3              0x3A5FCDAA
#define @ROYAL_BLUE_3            0x3A5FCDAA
#define @ROYALBLUE4              0x27408BAA
#define @ROYAL_BLUE_4            0x27408BAA
#define @BLUE1                   0x0000FFAA
#define @BLUE_1                  0x0000FFAA
#define @BLUE2                   0x0000EEAA
#define @BLUE_2                  0x0000EEAA
#define @BLUE3                   0x0000CDAA
#define @BLUE_3                  0x0000CDAA
#define @BLUE4                   0x00008BAA
#define @BLUE_4                  0x00008BAA
#define @DODGERBLUE1             0x1E90FFAA
#define @DODGER_BLUE_1           0x1E90FFAA
#define @DODGERBLUE2             0x1C86EEAA
#define @DODGER_BLUE_2           0x1C86EEAA
#define @DODGERBLUE3             0x1874CDAA
#define @DODGER_BLUE_3           0x1874CDAA
#define @DODGERBLUE4             0x104E8BAA
#define @DODGER_BLUE_4           0x104E8BAA
#define @STEELBLUE1              0x63B8FFAA
#define @STEEL_BLUE_1            0x63B8FFAA
#define @STEELBLUE2              0x5CACEEAA
#define @STEEL_BLUE_2            0x5CACEEAA
#define @STEELBLUE3              0x4F94CDAA
#define @STEEL_BLUE_3            0x4F94CDAA
#define @STEELBLUE4              0x36648BAA
#define @STEEL_BLUE_4            0x36648BAA
#define @DEEPSKYBLUE1            0x00BFFFAA
#define @DEEP_SKY_BLUE_1         0x00BFFFAA
#define @DEEPSKYBLUE2            0x00B2EEAA
#define @DEEP_SKY_BLUE_2         0x00B2EEAA
#define @DEEPSKYBLUE3            0x009ACDAA
#define @DEEP_SKY_BLUE_3         0x009ACDAA
#define @DEEPSKYBLUE4            0x00688BAA
#define @DEEP_SKY_BLUE_4         0x00688BAA
#define @SKYBLUE1                0x87CEFFAA
#define @SKY_BLUE_1              0x87CEFFAA
#define @SKYBLUE2                0x7EC0EEAA
#define @SKY_BLUE_2              0x7EC0EEAA
#define @SKYBLUE3                0x6CA6CDAA
#define @SKY_BLUE_3              0x6CA6CDAA
#define @SKYBLUE4                0x4A708BAA
#define @SKY_BLUE_4              0x4A708BAA
#define @LIGHTSKYBLUE1           0xB0E2FFAA
#define @LIGHT_SKY_BLUE_1        0xB0E2FFAA
#define @LIGHTSKYBLUE2           0xA4D3EEAA
#define @LIGHT_SKY_BLUE_2        0xA4D3EEAA
#define @LIGHTSKYBLUE3           0x8DB6CDAA
#define @LIGHT_SKY_BLUE_3        0x8DB6CDAA
#define @LIGHTSKYBLUE4           0x607B8BAA
#define @LIGHT_SKY_BLUE_4        0x607B8BAA
#define @SLATEGRAY1              0xC6E2FFAA
#define @SLATE_GRAY_1            0xC6E2FFAA
#define @SLATEGRAY2              0xB9D3EEAA
#define @SLATE_GRAY_2            0xB9D3EEAA
#define @SLATEGRAY3              0x9FB6CDAA
#define @SLATE_GRAY_3            0x9FB6CDAA
#define @SLATEGRAY4              0x6C7B8BAA
#define @SLATE_GRAY_4            0x6C7B8BAA
#define @SLATEGREY1              0xC6E2FFAA
#define @SLATE_GREY_1            0xC6E2FFAA
#define @SLATEGREY2              0xB9D3EEAA
#define @SLATE_GREY_2            0xB9D3EEAA
#define @SLATEGREY3              0x9FB6CDAA
#define @SLATE_GREY_3            0x9FB6CDAA
#define @SLATEGREY4              0x6C7B8BAA
#define @SLATE_GREY_4            0x6C7B8BAA
#define @LIGHTSTEELBLUE1         0xCAE1FFAA
#define @LIGHT_STEEL_BLUE_1      0xCAE1FFAA
#define @LIGHTSTEELBLUE2         0xBCD2EEAA
#define @LIGHT_STEEL_BLUE_2      0xBCD2EEAA
#define @LIGHTSTEELBLUE3         0xA2B5CDAA
#define @LIGHT_STEEL_BLUE_3      0xA2B5CDAA
#define @LIGHTSTEELBLUE4         0x6E7B8BAA
#define @LIGHT_STEEL_BLUE_4      0x6E7B8BAA
#define @LIGHTBLUE1              0xBFEFFFAA
#define @LIGHT_BLUE_1            0xBFEFFFAA
#define @LIGHTBLUE2              0xB2DFEEAA
#define @LIGHT_BLUE_2            0xB2DFEEAA
#define @LIGHTBLUE3              0x9AC0CDAA
#define @LIGHT_BLUE_3            0x9AC0CDAA
#define @LIGHTBLUE4              0x68838BAA
#define @LIGHT_BLUE_4            0x68838BAA
#define @LIGHTCYAN1              0xE0FFFFAA
#define @LIGHT_CYAN_1            0xE0FFFFAA
#define @LIGHTCYAN2              0xD1EEEEAA
#define @LIGHT_CYAN_2            0xD1EEEEAA
#define @LIGHTCYAN3              0xB4CDCDAA
#define @LIGHT_CYAN_3            0xB4CDCDAA
#define @LIGHTCYAN4              0x7A8B8BAA
#define @LIGHT_CYAN_4            0x7A8B8BAA
#define @PALETURQUOISE1          0xBBFFFFAA
#define @PALE_TURQUOISE_1        0xBBFFFFAA
#define @PALETURQUOISE2          0xAEEEEEAA
#define @PALE_TURQUOISE_2        0xAEEEEEAA
#define @PALETURQUOISE3          0x96CDCDAA
#define @PALE_TURQUOISE_3        0x96CDCDAA
#define @PALETURQUOISE4          0x668B8BAA
#define @PALE_TURQUOISE_4        0x668B8BAA
#define @CADETBLUE1              0x98F5FFAA
#define @CADET_BLUE_1            0x98F5FFAA
#define @CADETBLUE2              0x8EE5EEAA
#define @CADET_BLUE_2            0x8EE5EEAA
#define @CADETBLUE3              0x7AC5CDAA
#define @CADET_BLUE_3            0x7AC5CDAA
#define @CADETBLUE4              0x53868BAA
#define @CADET_BLUE_4            0x53868BAA
#define @TURQUOISE1              0x00F5FFAA
#define @TURQUOISE_1             0x00F5FFAA
#define @TURQUOISE2              0x00E5EEAA
#define @TURQUOISE_2             0x00E5EEAA
#define @TURQUOISE3              0x00C5CDAA
#define @TURQUOISE_3             0x00C5CDAA
#define @TURQUOISE4              0x00868BAA
#define @TURQUOISE_4             0x00868BAA
#define @CYAN1                   0x00FFFFAA
#define @CYAN_1                  0x00FFFFAA
#define @CYAN2                   0x00EEEEAA
#define @CYAN_2                  0x00EEEEAA
#define @CYAN3                   0x00CDCDAA
#define @CYAN_3                  0x00CDCDAA
#define @CYAN4                   0x008B8BAA
#define @CYAN_4                  0x008B8BAA
#define @DARKSLATEGRAY1          0x97FFFFAA
#define @DARK_SLATE_GRAY_1       0x97FFFFAA
#define @DARKSLATEGRAY2          0x8DEEEEAA
#define @DARK_SLATE_GRAY_2       0x8DEEEEAA
#define @DARKSLATEGRAY3          0x79CDCDAA
#define @DARK_SLATE_GRAY_3       0x79CDCDAA
#define @DARKSLATEGRAY4          0x528B8BAA
#define @DARK_SLATE_GRAY_4       0x528B8BAA
#define @DARKSLATEGREY1          0x97FFFFAA
#define @DARK_SLATE_GREY_1       0x97FFFFAA
#define @DARKSLATEGREY2          0x8DEEEEAA
#define @DARK_SLATE_GREY_2       0x8DEEEEAA
#define @DARKSLATEGREY3          0x79CDCDAA
#define @DARK_SLATE_GREY_3       0x79CDCDAA
#define @DARKSLATEGREY4          0x528B8BAA
#define @DARK_SLATE_GREY_4       0x528B8BAA
#define @AQUAMARINE1             0x7FFFD4AA
#define @AQUAMARINE_1            0x7FFFD4AA
#define @AQUAMARINE2             0x76EEC6AA
#define @AQUAMARINE_2            0x76EEC6AA
#define @AQUAMARINE3             0x66CDAAAA
#define @AQUAMARINE_3            0x66CDAAAA
#define @AQUAMARINE4             0x458B74AA
#define @AQUAMARINE_4            0x458B74AA
#define @DARKSEAGREEN1           0xC1FFC1AA
#define @DARK_SEA_GREEN_1        0xC1FFC1AA
#define @DARKSEAGREEN2           0xB4EEB4AA
#define @DARK_SEA_GREEN_2        0xB4EEB4AA
#define @DARKSEAGREEN3           0x9BCD9BAA
#define @DARK_SEA_GREEN_3        0x9BCD9BAA
#define @DARKSEAGREEN4           0x698B69AA
#define @DARK_SEA_GREEN_4        0x698B69AA
#define @SEAGREEN1               0x54FF9FAA
#define @SEA_GREEN_1             0x54FF9FAA
#define @SEAGREEN2               0x4EEE94AA
#define @SEA_GREEN_2             0x4EEE94AA
#define @SEAGREEN3               0x43CD80AA
#define @SEA_GREEN_3             0x43CD80AA
#define @SEAGREEN4               0x2E8B57AA
#define @SEA_GREEN_4             0x2E8B57AA
#define @PALEGREEN1              0x9AFF9AAA
#define @PALE_GREEN_1            0x9AFF9AAA
#define @PALEGREEN2              0x90EE90AA
#define @PALE_GREEN_2            0x90EE90AA
#define @PALEGREEN3              0x7CCD7CAA
#define @PALE_GREEN_3            0x7CCD7CAA
#define @PALEGREEN4              0x548B54AA
#define @PALE_GREEN_4            0x548B54AA
#define @SPRINGGREEN1            0x00FF7FAA
#define @SPRING_GREEN_1          0x00FF7FAA
#define @SPRINGGREEN2            0x00EE76AA
#define @SPRING_GREEN_2          0x00EE76AA
#define @SPRINGGREEN3            0x00CD66AA
#define @SPRING_GREEN_3          0x00CD66AA
#define @SPRINGGREEN4            0x008B45AA
#define @SPRING_GREEN_4          0x008B45AA
#define @GREEN1                  0x00FF00AA
#define @GREEN_1                 0x00FF00AA
#define @GREEN2                  0x00EE00AA
#define @GREEN_2                 0x00EE00AA
#define @GREEN3                  0x00CD00AA
#define @GREEN_3                 0x00CD00AA
#define @GREEN4                  0x008B00AA
#define @GREEN_4                 0x008B00AA
#define @CHARTREUSE1             0x7FFF00AA
#define @CHARTREUSE_1            0x7FFF00AA
#define @CHARTREUSE2             0x76EE00AA
#define @CHARTREUSE_2            0x76EE00AA
#define @CHARTREUSE3             0x66CD00AA
#define @CHARTREUSE_3            0x66CD00AA
#define @CHARTREUSE4             0x458B00AA
#define @CHARTREUSE_4            0x458B00AA
#define @OLIVEDRAB1              0xC0FF3EAA
#define @OLIVE_DRAB_1            0xC0FF3EAA
#define @OLIVEDRAB2              0xB3EE3AAA
#define @OLIVE_DRAB_2            0xB3EE3AAA
#define @OLIVEDRAB3              0x9ACD32AA
#define @OLIVE_DRAB_3            0x9ACD32AA
#define @OLIVEDRAB4              0x698B22AA
#define @OLIVE_DRAB_4            0x698B22AA
#define @DARKOLIVEGREEN1         0xCAFF70AA
#define @DARK_OLIVE_GREEN_1      0xCAFF70AA
#define @DARKOLIVEGREEN2         0xBCEE68AA
#define @DARK_OLIVE_GREEN_2      0xBCEE68AA
#define @DARKOLIVEGREEN3         0xA2CD5AAA
#define @DARK_OLIVE_GREEN_3      0xA2CD5AAA
#define @DARKOLIVEGREEN4         0x6E8B3DAA
#define @DARK_OLIVE_GREEN_4      0x6E8B3DAA
#define @KHAKI1                  0xFFF68FAA
#define @KHAKI_1                 0xFFF68FAA
#define @KHAKI2                  0xEEE685AA
#define @KHAKI_2                 0xEEE685AA
#define @KHAKI3                  0xCDC673AA
#define @KHAKI_3                 0xCDC673AA
#define @KHAKI4                  0x8B864EAA
#define @KHAKI_4                 0x8B864EAA
#define @LIGHTGOLDENROD1         0xFFEC8BAA
#define @LIGHT_GOLDENROD_1       0xFFEC8BAA
#define @LIGHTGOLDENROD2         0xEEDC82AA
#define @LIGHT_GOLDENROD_2       0xEEDC82AA
#define @LIGHTGOLDENROD3         0xCDBE70AA
#define @LIGHT_GOLDENROD_3       0xCDBE70AA
#define @LIGHTGOLDENROD4         0x8B814CAA
#define @LIGHT_GOLDENROD_4       0x8B814CAA
#define @LIGHTYELLOW1            0xFFFFE0AA
#define @LIGHT_YELLOW_1          0xFFFFE0AA
#define @LIGHTYELLOW2            0xEEEED1AA
#define @LIGHT_YELLOW_2          0xEEEED1AA
#define @LIGHTYELLOW3            0xCDCDB4AA
#define @LIGHT_YELLOW_3          0xCDCDB4AA
#define @LIGHTYELLOW4            0x8B8B7AAA
#define @LIGHT_YELLOW_4          0x8B8B7AAA
#define @YELLOW1                 0xFFFF00AA
#define @YELLOW_1                0xFFFF00AA
#define @YELLOW2                 0xEEEE00AA
#define @YELLOW_2                0xEEEE00AA
#define @YELLOW3                 0xCDCD00AA
#define @YELLOW_3                0xCDCD00AA
#define @YELLOW4                 0x8B8B00AA
#define @YELLOW_4                0x8B8B00AA
#define @GOLD1                   0xFFD700AA
#define @GOLD_1                  0xFFD700AA
#define @GOLD2                   0xEEC900AA
#define @GOLD_2                  0xEEC900AA
#define @GOLD3                   0xCDAD00AA
#define @GOLD_3                  0xCDAD00AA
#define @GOLD4                   0x8B7500AA
#define @GOLD_4                  0x8B7500AA
#define @GOLDENROD1              0xFFC125AA
#define @GOLDENROD_1             0xFFC125AA
#define @GOLDENROD2              0xEEB422AA
#define @GOLDENROD_2             0xEEB422AA
#define @GOLDENROD3              0xCD9B1DAA
#define @GOLDENROD_3             0xCD9B1DAA
#define @GOLDENROD4              0x8B6914AA
#define @GOLDENROD_4             0x8B6914AA
#define @DARKGOLDENROD1          0xFFB90FAA
#define @DARK_GOLDENROD_1        0xFFB90FAA
#define @DARKGOLDENROD2          0xEEAD0EAA
#define @DARK_GOLDENROD_2        0xEEAD0EAA
#define @DARKGOLDENROD3          0xCD950CAA
#define @DARK_GOLDENROD_3        0xCD950CAA
#define @DARKGOLDENROD4          0x8B6508AA
#define @DARK_GOLDENROD_4        0x8B6508AA
#define @ROSYBROWN1              0xFFC1C1AA
#define @ROSY_BROWN_1            0xFFC1C1AA
#define @ROSYBROWN2              0xEEB4B4AA
#define @ROSY_BROWN_2            0xEEB4B4AA
#define @ROSYBROWN3              0xCD9B9BAA
#define @ROSY_BROWN_3            0xCD9B9BAA
#define @ROSYBROWN4              0x8B6969AA
#define @ROSY_BROWN_4            0x8B6969AA
#define @INDIANRED1              0xFF6A6AAA
#define @INDIAN_RED_1            0xFF6A6AAA
#define @INDIANRED2              0xEE6363AA
#define @INDIAN_RED_2            0xEE6363AA
#define @INDIANRED3              0xCD5555AA
#define @INDIAN_RED_3            0xCD5555AA
#define @INDIANRED4              0x8B3A3AAA
#define @INDIAN_RED_4            0x8B3A3AAA
#define @SIENNA1                 0xFF8247AA
#define @SIENNA_1                0xFF8247AA
#define @SIENNA2                 0xEE7942AA
#define @SIENNA_2                0xEE7942AA
#define @SIENNA3                 0xCD6839AA
#define @SIENNA_3                0xCD6839AA
#define @SIENNA4                 0x8B4726AA
#define @SIENNA_4                0x8B4726AA
#define @BURLYWOOD1              0xFFD39BAA
#define @BURLYWOOD_1             0xFFD39BAA
#define @BURLYWOOD2              0xEEC591AA
#define @BURLYWOOD_2             0xEEC591AA
#define @BURLYWOOD3              0xCDAA7DAA
#define @BURLYWOOD_3             0xCDAA7DAA
#define @BURLYWOOD4              0x8B7355AA
#define @BURLYWOOD_4             0x8B7355AA
#define @WHEAT1                  0xFFE7BAAA
#define @WHEAT_1                 0xFFE7BAAA
#define @WHEAT2                  0xEED8AEAA
#define @WHEAT_2                 0xEED8AEAA
#define @WHEAT3                  0xCDBA96AA
#define @WHEAT_3                 0xCDBA96AA
#define @WHEAT4                  0x8B7E66AA
#define @WHEAT_4                 0x8B7E66AA
#define @TAN1                    0xFFA54FAA
#define @TAN_1                   0xFFA54FAA
#define @TAN2                    0xEE9A49AA
#define @TAN_2                   0xEE9A49AA
#define @TAN3                    0xCD853FAA
#define @TAN_3                   0xCD853FAA
#define @TAN4                    0x8B5A2BAA
#define @TAN_4                   0x8B5A2BAA
#define @CHOCOLATE1              0xFF7F24AA
#define @CHOCOLATE_1             0xFF7F24AA
#define @CHOCOLATE2              0xEE7621AA
#define @CHOCOLATE_2             0xEE7621AA
#define @CHOCOLATE3              0xCD661DAA
#define @CHOCOLATE_3             0xCD661DAA
#define @CHOCOLATE4              0x8B4513AA
#define @CHOCOLATE_4             0x8B4513AA
#define @FIREBRICK1              0xFF3030AA
#define @FIREBRICK_1             0xFF3030AA
#define @FIREBRICK2              0xEE2C2CAA
#define @FIREBRICK_2             0xEE2C2CAA
#define @FIREBRICK3              0xCD2626AA
#define @FIREBRICK_3             0xCD2626AA
#define @FIREBRICK4              0x8B1A1AAA
#define @FIREBRICK_4             0x8B1A1AAA
#define @BROWN1                  0xFF4040AA
#define @BROWN_1                 0xFF4040AA
#define @BROWN2                  0xEE3B3BAA
#define @BROWN_2                 0xEE3B3BAA
#define @BROWN3                  0xCD3333AA
#define @BROWN_3                 0xCD3333AA
#define @BROWN4                  0x8B2323AA
#define @BROWN_4                 0x8B2323AA
#define @SALMON1                 0xFF8C69AA
#define @SALMON_1                0xFF8C69AA
#define @SALMON2                 0xEE8262AA
#define @SALMON_2                0xEE8262AA
#define @SALMON3                 0xCD7054AA
#define @SALMON_3                0xCD7054AA
#define @SALMON4                 0x8B4C39AA
#define @SALMON_4                0x8B4C39AA
#define @LIGHTSALMON1            0xFFA07AAA
#define @LIGHT_SALMON_1          0xFFA07AAA
#define @LIGHTSALMON2            0xEE9572AA
#define @LIGHT_SALMON_2          0xEE9572AA
#define @LIGHTSALMON3            0xCD8162AA
#define @LIGHT_SALMON_3          0xCD8162AA
#define @LIGHTSALMON4            0x8B5742AA
#define @LIGHT_SALMON_4          0x8B5742AA
#define @ORANGE1                 0xFFA500AA
#define @ORANGE_1                0xFFA500AA
#define @ORANGE2                 0xEE9A00AA
#define @ORANGE_2                0xEE9A00AA
#define @ORANGE3                 0xCD8500AA
#define @ORANGE_3                0xCD8500AA
#define @ORANGE4                 0x8B5A00AA
#define @ORANGE_4                0x8B5A00AA
#define @DARKORANGE1             0xFF7F00AA
#define @DARK_ORANGE_1           0xFF7F00AA
#define @DARKORANGE2             0xEE7600AA
#define @DARK_ORANGE_2           0xEE7600AA
#define @DARKORANGE3             0xCD6600AA
#define @DARK_ORANGE_3           0xCD6600AA
#define @DARKORANGE4             0x8B4500AA
#define @DARK_ORANGE_4           0x8B4500AA
#define @CORAL1                  0xFF7256AA
#define @CORAL_1                 0xFF7256AA
#define @CORAL2                  0xEE6A50AA
#define @CORAL_2                 0xEE6A50AA
#define @CORAL3                  0xCD5B45AA
#define @CORAL_3                 0xCD5B45AA
#define @CORAL4                  0x8B3E2FAA
#define @CORAL_4                 0x8B3E2FAA
#define @TOMATO1                 0xFF6347AA
#define @TOMATO_1                0xFF6347AA
#define @TOMATO2                 0xEE5C42AA
#define @TOMATO_2                0xEE5C42AA
#define @TOMATO3                 0xCD4F39AA
#define @TOMATO_3                0xCD4F39AA
#define @TOMATO4                 0x8B3626AA
#define @TOMATO_4                0x8B3626AA
#define @ORANGERED1              0xFF4500AA
#define @ORANGE_RED_1            0xFF4500AA
#define @ORANGERED2              0xEE4000AA
#define @ORANGE_RED_2            0xEE4000AA
#define @ORANGERED3              0xCD3700AA
#define @ORANGE_RED_3            0xCD3700AA
#define @ORANGERED4              0x8B2500AA
#define @ORANGE_RED_4            0x8B2500AA
#define @RED1                    0xFF0000AA
#define @RED_1                   0xFF0000AA
#define @RED2                    0xEE0000AA
#define @RED_2                   0xEE0000AA
#define @RED3                    0xCD0000AA
#define @RED_3                   0xCD0000AA
#define @RED4                    0x8B0000AA
#define @RED_4                   0x8B0000AA
#define @DEEPPINK1               0xFF1493AA
#define @DEEP_PINK_1             0xFF1493AA
#define @DEEPPINK2               0xEE1289AA
#define @DEEP_PINK_2             0xEE1289AA
#define @DEEPPINK3               0xCD1076AA
#define @DEEP_PINK_3             0xCD1076AA
#define @DEEPPINK4               0x8B0A50AA
#define @DEEP_PINK_4             0x8B0A50AA
#define @HOTPINK1                0xFF6EB4AA
#define @HOT_PINK_1              0xFF6EB4AA
#define @HOTPINK2                0xEE6AA7AA
#define @HOT_PINK_2              0xEE6AA7AA
#define @HOTPINK3                0xCD6090AA
#define @HOT_PINK_3              0xCD6090AA
#define @HOTPINK4                0x8B3A62AA
#define @HOT_PINK_4              0x8B3A62AA
#define @PINK1                   0xFFB5C5AA
#define @PINK_1                  0xFFB5C5AA
#define @PINK2                   0xEEA9B8AA
#define @PINK_2                  0xEEA9B8AA
#define @PINK3                   0xCD919EAA
#define @PINK_3                  0xCD919EAA
#define @PINK4                   0x8B636CAA
#define @PINK_4                  0x8B636CAA
#define @LIGHTPINK1              0xFFAEB9AA
#define @LIGHT_PINK_1            0xFFAEB9AA
#define @LIGHTPINK2              0xEEA2ADAA
#define @LIGHT_PINK_2            0xEEA2ADAA
#define @LIGHTPINK3              0xCD8C95AA
#define @LIGHT_PINK_3            0xCD8C95AA
#define @LIGHTPINK4              0x8B5F65AA
#define @LIGHT_PINK_4            0x8B5F65AA
#define @PALEVIOLETRED1          0xFF82ABAA
#define @PALE_VIOLET_RED_1       0xFF82ABAA
#define @PALEVIOLETRED2          0xEE799FAA
#define @PALE_VIOLET_RED_2       0xEE799FAA
#define @PALEVIOLETRED3          0xCD6889AA
#define @PALE_VIOLET_RED_3       0xCD6889AA
#define @PALEVIOLETRED4          0x8B475DAA
#define @PALE_VIOLET_RED_4       0x8B475DAA
#define @MAROON1                 0xFF34B3AA
#define @MAROON_1                0xFF34B3AA
#define @MAROON2                 0xEE30A7AA
#define @MAROON_2                0xEE30A7AA
#define @MAROON3                 0xCD2990AA
#define @MAROON_3                0xCD2990AA
#define @MAROON4                 0x8B1C62AA
#define @MAROON_4                0x8B1C62AA
#define @VIOLETRED1              0xFF3E96AA
#define @VIOLET_RED_1            0xFF3E96AA
#define @VIOLETRED2              0xEE3A8CAA
#define @VIOLET_RED_2            0xEE3A8CAA
#define @VIOLETRED3              0xCD3278AA
#define @VIOLET_RED_3            0xCD3278AA
#define @VIOLETRED4              0x8B2252AA
#define @VIOLET_RED_4            0x8B2252AA
#define @MAGENTA1                0xFF00FFAA
#define @MAGENTA_1               0xFF00FFAA
#define @MAGENTA2                0xEE00EEAA
#define @MAGENTA_2               0xEE00EEAA
#define @MAGENTA3                0xCD00CDAA
#define @MAGENTA_3               0xCD00CDAA
#define @MAGENTA4                0x8B008BAA
#define @MAGENTA_4               0x8B008BAA
#define @ORCHID1                 0xFF83FAAA
#define @ORCHID_1                0xFF83FAAA
#define @ORCHID2                 0xEE7AE9AA
#define @ORCHID_2                0xEE7AE9AA
#define @ORCHID3                 0xCD69C9AA
#define @ORCHID_3                0xCD69C9AA
#define @ORCHID4                 0x8B4789AA
#define @ORCHID_4                0x8B4789AA
#define @PLUM1                   0xFFBBFFAA
#define @PLUM_1                  0xFFBBFFAA
#define @PLUM2                   0xEEAEEEAA
#define @PLUM_2                  0xEEAEEEAA
#define @PLUM3                   0xCD96CDAA
#define @PLUM_3                  0xCD96CDAA
#define @PLUM4                   0x8B668BAA
#define @PLUM_4                  0x8B668BAA
#define @MEDIUMORCHID1           0xE066FFAA
#define @MEDIUM_ORCHID_1         0xE066FFAA
#define @MEDIUMORCHID2           0xD15FEEAA
#define @MEDIUM_ORCHID_2         0xD15FEEAA
#define @MEDIUMORCHID3           0xB452CDAA
#define @MEDIUM_ORCHID_3         0xB452CDAA
#define @MEDIUMORCHID4           0x7A378BAA
#define @MEDIUM_ORCHID_4         0x7A378BAA
#define @DARKORCHID1             0xBF3EFFAA
#define @DARK_ORCHID_1           0xBF3EFFAA
#define @DARKORCHID2             0xB23AEEAA
#define @DARK_ORCHID_2           0xB23AEEAA
#define @DARKORCHID3             0x9A32CDAA
#define @DARK_ORCHID_3           0x9A32CDAA
#define @DARKORCHID4             0x68228BAA
#define @DARK_ORCHID_4           0x68228BAA
#define @PURPLE1                 0x9B30FFAA
#define @PURPLE_1                0x9B30FFAA
#define @PURPLE2                 0x912CEEAA
#define @PURPLE_2                0x912CEEAA
#define @PURPLE3                 0x7D26CDAA
#define @PURPLE_3                0x7D26CDAA
#define @PURPLE4                 0x551A8BAA
#define @PURPLE_4                0x551A8BAA
#define @MEDIUMPURPLE1           0xAB82FFAA
#define @MEDIUM_PURPLE_1         0xAB82FFAA
#define @MEDIUMPURPLE2           0x9F79EEAA
#define @MEDIUM_PURPLE_2         0x9F79EEAA
#define @MEDIUMPURPLE3           0x8968CDAA
#define @MEDIUM_PURPLE_3         0x8968CDAA
#define @MEDIUMPURPLE4           0x5D478BAA
#define @MEDIUM_PURPLE_4         0x5D478BAA
#define @THISTLE1                0xFFE1FFAA
#define @THISTLE_1               0xFFE1FFAA
#define @THISTLE2                0xEED2EEAA
#define @THISTLE_2               0xEED2EEAA
#define @THISTLE3                0xCDB5CDAA
#define @THISTLE_3               0xCDB5CDAA
#define @THISTLE4                0x8B7B8BAA
#define @THISTLE_4               0x8B7B8BAA
#define @GRAY0                   0x000000AA
#define @GRAY_0                  0x000000AA
#define @GREY0                   0x000000AA
#define @GREY_0                  0x000000AA
#define @GRAY1                   0x030303AA
#define @GRAY_1                  0x030303AA
#define @GREY1                   0x030303AA
#define @GREY_1                  0x030303AA
#define @GRAY2                   0x050505AA
#define @GRAY_2                  0x050505AA
#define @GREY2                   0x050505AA
#define @GREY_2                  0x050505AA
#define @GRAY3                   0x080808AA
#define @GRAY_3                  0x080808AA
#define @GREY3                   0x080808AA
#define @GREY_3                  0x080808AA
#define @GRAY4                   0x0A0A0AAA
#define @GRAY_4                  0x0A0A0AAA
#define @GREY4                   0x0A0A0AAA
#define @GREY_4                  0x0A0A0AAA
#define @GRAY5                   0x0D0D0DAA
#define @GRAY_5                  0x0D0D0DAA
#define @GREY5                   0x0D0D0DAA
#define @GREY_5                  0x0D0D0DAA
#define @GRAY6                   0x0F0F0FAA
#define @GRAY_6                  0x0F0F0FAA
#define @GREY6                   0x0F0F0FAA
#define @GREY_6                  0x0F0F0FAA
#define @GRAY7                   0x121212AA
#define @GRAY_7                  0x121212AA
#define @GREY7                   0x121212AA
#define @GREY_7                  0x121212AA
#define @GRAY8                   0x141414AA
#define @GRAY_8                  0x141414AA
#define @GREY8                   0x141414AA
#define @GREY_8                  0x141414AA
#define @GRAY9                   0x171717AA
#define @GRAY_9                  0x171717AA
#define @GREY9                   0x171717AA
#define @GREY_9                  0x171717AA
#define @GRAY10                  0x1A1A1AAA
#define @GRAY_10                 0x1A1A1AAA
#define @GREY10                  0x1A1A1AAA
#define @GREY_10                 0x1A1A1AAA
#define @GRAY11                  0x1C1C1CAA
#define @GRAY_11                 0x1C1C1CAA
#define @GREY11                  0x1C1C1CAA
#define @GREY_11                 0x1C1C1CAA
#define @GRAY12                  0x1F1F1FAA
#define @GRAY_12                 0x1F1F1FAA
#define @GREY12                  0x1F1F1FAA
#define @GREY_12                 0x1F1F1FAA
#define @GRAY13                  0x212121AA
#define @GRAY_13                 0x212121AA
#define @GREY13                  0x212121AA
#define @GREY_13                 0x212121AA
#define @GRAY14                  0x242424AA
#define @GRAY_14                 0x242424AA
#define @GREY14                  0x242424AA
#define @GREY_14                 0x242424AA
#define @GRAY15                  0x262626AA
#define @GRAY_15                 0x262626AA
#define @GREY15                  0x262626AA
#define @GREY_15                 0x262626AA
#define @GRAY16                  0x292929AA
#define @GRAY_16                 0x292929AA
#define @GREY16                  0x292929AA
#define @GREY_16                 0x292929AA
#define @GRAY17                  0x2B2B2BAA
#define @GRAY_17                 0x2B2B2BAA
#define @GREY17                  0x2B2B2BAA
#define @GREY_17                 0x2B2B2BAA
#define @GRAY18                  0x2E2E2EAA
#define @GRAY_18                 0x2E2E2EAA
#define @GREY18                  0x2E2E2EAA
#define @GREY_18                 0x2E2E2EAA
#define @GRAY19                  0x303030AA
#define @GRAY_19                 0x303030AA
#define @GREY19                  0x303030AA
#define @GREY_19                 0x303030AA
#define @GRAY20                  0x333333AA
#define @GRAY_20                 0x333333AA
#define @GREY20                  0x333333AA
#define @GREY_20                 0x333333AA
#define @GRAY21                  0x363636AA
#define @GRAY_21                 0x363636AA
#define @GREY21                  0x363636AA
#define @GREY_21                 0x363636AA
#define @GRAY22                  0x383838AA
#define @GRAY_22                 0x383838AA
#define @GREY22                  0x383838AA
#define @GREY_22                 0x383838AA
#define @GRAY23                  0x3B3B3BAA
#define @GRAY_23                 0x3B3B3BAA
#define @GREY23                  0x3B3B3BAA
#define @GREY_23                 0x3B3B3BAA
#define @GRAY24                  0x3D3D3DAA
#define @GRAY_24                 0x3D3D3DAA
#define @GREY24                  0x3D3D3DAA
#define @GREY_24                 0x3D3D3DAA
#define @GRAY25                  0x404040AA
#define @GRAY_25                 0x404040AA
#define @GREY25                  0x404040AA
#define @GREY_25                 0x404040AA
#define @GRAY26                  0x424242AA
#define @GRAY_26                 0x424242AA
#define @GREY26                  0x424242AA
#define @GREY_26                 0x424242AA
#define @GRAY27                  0x454545AA
#define @GRAY_27                 0x454545AA
#define @GREY27                  0x454545AA
#define @GREY_27                 0x454545AA
#define @GRAY28                  0x474747AA
#define @GRAY_28                 0x474747AA
#define @GREY28                  0x474747AA
#define @GREY_28                 0x474747AA
#define @GRAY29                  0x4A4A4AAA
#define @GRAY_29                 0x4A4A4AAA
#define @GREY29                  0x4A4A4AAA
#define @GREY_29                 0x4A4A4AAA
#define @GRAY30                  0x4D4D4DAA
#define @GRAY_30                 0x4D4D4DAA
#define @GREY30                  0x4D4D4DAA
#define @GREY_30                 0x4D4D4DAA
#define @GRAY31                  0x4F4F4FAA
#define @GRAY_31                 0x4F4F4FAA
#define @GREY31                  0x4F4F4FAA
#define @GREY_31                 0x4F4F4FAA
#define @GRAY32                  0x525252AA
#define @GRAY_32                 0x525252AA
#define @GREY32                  0x525252AA
#define @GREY_32                 0x525252AA
#define @GRAY33                  0x545454AA
#define @GRAY_33                 0x545454AA
#define @GREY33                  0x545454AA
#define @GREY_33                 0x545454AA
#define @GRAY34                  0x575757AA
#define @GRAY_34                 0x575757AA
#define @GREY34                  0x575757AA
#define @GREY_34                 0x575757AA
#define @GRAY35                  0x595959AA
#define @GRAY_35                 0x595959AA
#define @GREY35                  0x595959AA
#define @GREY_35                 0x595959AA
#define @GRAY36                  0x5C5C5CAA
#define @GRAY_36                 0x5C5C5CAA
#define @GREY36                  0x5C5C5CAA
#define @GREY_36                 0x5C5C5CAA
#define @GRAY37                  0x5E5E5EAA
#define @GRAY_37                 0x5E5E5EAA
#define @GREY37                  0x5E5E5EAA
#define @GREY_37                 0x5E5E5EAA
#define @GRAY38                  0x616161AA
#define @GRAY_38                 0x616161AA
#define @GREY38                  0x616161AA
#define @GREY_38                 0x616161AA
#define @GRAY39                  0x636363AA
#define @GRAY_39                 0x636363AA
#define @GREY39                  0x636363AA
#define @GREY_39                 0x636363AA
#define @GRAY40                  0x666666AA
#define @GRAY_40                 0x666666AA
#define @GREY40                  0x666666AA
#define @GREY_40                 0x666666AA
#define @GRAY41                  0x696969AA
#define @GRAY_41                 0x696969AA
#define @GREY41                  0x696969AA
#define @GREY_41                 0x696969AA
#define @GRAY42                  0x6B6B6BAA
#define @GRAY_42                 0x6B6B6BAA
#define @GREY42                  0x6B6B6BAA
#define @GREY_42                 0x6B6B6BAA
#define @GRAY43                  0x6E6E6EAA
#define @GRAY_43                 0x6E6E6EAA
#define @GREY43                  0x6E6E6EAA
#define @GREY_43                 0x6E6E6EAA
#define @GRAY44                  0x707070AA
#define @GRAY_44                 0x707070AA
#define @GREY44                  0x707070AA
#define @GREY_44                 0x707070AA
#define @GRAY45                  0x737373AA
#define @GRAY_45                 0x737373AA
#define @GREY45                  0x737373AA
#define @GREY_45                 0x737373AA
#define @GRAY46                  0x757575AA
#define @GRAY_46                 0x757575AA
#define @GREY46                  0x757575AA
#define @GREY_46                 0x757575AA
#define @GRAY47                  0x787878AA
#define @GRAY_47                 0x787878AA
#define @GREY47                  0x787878AA
#define @GREY_47                 0x787878AA
#define @GRAY48                  0x7A7A7AAA
#define @GRAY_48                 0x7A7A7AAA
#define @GREY48                  0x7A7A7AAA
#define @GREY_48                 0x7A7A7AAA
#define @GRAY49                  0x7D7D7DAA
#define @GRAY_49                 0x7D7D7DAA
#define @GREY49                  0x7D7D7DAA
#define @GREY_49                 0x7D7D7DAA
#define @GRAY50                  0x7F7F7FAA
#define @GRAY_50                 0x7F7F7FAA
#define @GREY50                  0x7F7F7FAA
#define @GREY_50                 0x7F7F7FAA
#define @GRAY51                  0x828282AA
#define @GRAY_51                 0x828282AA
#define @GREY51                  0x828282AA
#define @GREY_51                 0x828282AA
#define @GRAY52                  0x858585AA
#define @GRAY_52                 0x858585AA
#define @GREY52                  0x858585AA
#define @GREY_52                 0x858585AA
#define @GRAY53                  0x878787AA
#define @GRAY_53                 0x878787AA
#define @GREY53                  0x878787AA
#define @GREY_53                 0x878787AA
#define @GRAY54                  0x8A8A8AAA
#define @GRAY_54                 0x8A8A8AAA
#define @GREY54                  0x8A8A8AAA
#define @GREY_54                 0x8A8A8AAA
#define @GRAY55                  0x8C8C8CAA
#define @GRAY_55                 0x8C8C8CAA
#define @GREY55                  0x8C8C8CAA
#define @GREY_55                 0x8C8C8CAA
#define @GRAY56                  0x8F8F8FAA
#define @GRAY_56                 0x8F8F8FAA
#define @GREY56                  0x8F8F8FAA
#define @GREY_56                 0x8F8F8FAA
#define @GRAY57                  0x919191AA
#define @GRAY_57                 0x919191AA
#define @GREY57                  0x919191AA
#define @GREY_57                 0x919191AA
#define @GRAY58                  0x949494AA
#define @GRAY_58                 0x949494AA
#define @GREY58                  0x949494AA
#define @GREY_58                 0x949494AA
#define @GRAY59                  0x969696AA
#define @GRAY_59                 0x969696AA
#define @GREY59                  0x969696AA
#define @GREY_59                 0x969696AA
#define @GRAY60                  0x999999AA
#define @GRAY_60                 0x999999AA
#define @GREY60                  0x999999AA
#define @GREY_60                 0x999999AA
#define @GRAY61                  0x9C9C9CAA
#define @GRAY_61                 0x9C9C9CAA
#define @GREY61                  0x9C9C9CAA
#define @GREY_61                 0x9C9C9CAA
#define @GRAY62                  0x9E9E9EAA
#define @GRAY_62                 0x9E9E9EAA
#define @GREY62                  0x9E9E9EAA
#define @GREY_62                 0x9E9E9EAA
#define @GRAY63                  0xA1A1A1AA
#define @GRAY_63                 0xA1A1A1AA
#define @GREY63                  0xA1A1A1AA
#define @GREY_63                 0xA1A1A1AA
#define @GRAY64                  0xA3A3A3AA
#define @GRAY_64                 0xA3A3A3AA
#define @GREY64                  0xA3A3A3AA
#define @GREY_64                 0xA3A3A3AA
#define @GRAY65                  0xA6A6A6AA
#define @GRAY_65                 0xA6A6A6AA
#define @GREY65                  0xA6A6A6AA
#define @GREY_65                 0xA6A6A6AA
#define @GRAY66                  0xA8A8A8AA
#define @GRAY_66                 0xA8A8A8AA
#define @GREY66                  0xA8A8A8AA
#define @GREY_66                 0xA8A8A8AA
#define @GRAY67                  0xABABABAA
#define @GRAY_67                 0xABABABAA
#define @GREY67                  0xABABABAA
#define @GREY_67                 0xABABABAA
#define @GRAY68                  0xADADADAA
#define @GRAY_68                 0xADADADAA
#define @GREY68                  0xADADADAA
#define @GREY_68                 0xADADADAA
#define @GRAY69                  0xB0B0B0AA
#define @GRAY_69                 0xB0B0B0AA
#define @GREY69                  0xB0B0B0AA
#define @GREY_69                 0xB0B0B0AA
#define @GRAY70                  0xB3B3B3AA
#define @GRAY_70                 0xB3B3B3AA
#define @GREY70                  0xB3B3B3AA
#define @GREY_70                 0xB3B3B3AA
#define @GRAY71                  0xB5B5B5AA
#define @GRAY_71                 0xB5B5B5AA
#define @GREY71                  0xB5B5B5AA
#define @GREY_71                 0xB5B5B5AA
#define @GRAY72                  0xB8B8B8AA
#define @GRAY_72                 0xB8B8B8AA
#define @GREY72                  0xB8B8B8AA
#define @GREY_72                 0xB8B8B8AA
#define @GRAY73                  0xBABABAAA
#define @GRAY_73                 0xBABABAAA
#define @GREY73                  0xBABABAAA
#define @GREY_73                 0xBABABAAA
#define @GRAY74                  0xBDBDBDAA
#define @GRAY_74                 0xBDBDBDAA
#define @GREY74                  0xBDBDBDAA
#define @GREY_74                 0xBDBDBDAA
#define @GRAY75                  0xBFBFBFAA
#define @GRAY_75                 0xBFBFBFAA
#define @GREY75                  0xBFBFBFAA
#define @GREY_75                 0xBFBFBFAA
#define @GRAY76                  0xC2C2C2AA
#define @GRAY_76                 0xC2C2C2AA
#define @GREY76                  0xC2C2C2AA
#define @GREY_76                 0xC2C2C2AA
#define @GRAY77                  0xC4C4C4AA
#define @GRAY_77                 0xC4C4C4AA
#define @GREY77                  0xC4C4C4AA
#define @GREY_77                 0xC4C4C4AA
#define @GRAY78                  0xC7C7C7AA
#define @GRAY_78                 0xC7C7C7AA
#define @GREY78                  0xC7C7C7AA
#define @GREY_78                 0xC7C7C7AA
#define @GRAY79                  0xC9C9C9AA
#define @GRAY_79                 0xC9C9C9AA
#define @GREY79                  0xC9C9C9AA
#define @GREY_79                 0xC9C9C9AA
#define @GRAY80                  0xCCCCCCAA
#define @GRAY_80                 0xCCCCCCAA
#define @GREY80                  0xCCCCCCAA
#define @GREY_80                 0xCCCCCCAA
#define @GRAY81                  0xCFCFCFAA
#define @GRAY_81                 0xCFCFCFAA
#define @GREY81                  0xCFCFCFAA
#define @GREY_81                 0xCFCFCFAA
#define @GRAY82                  0xD1D1D1AA
#define @GRAY_82                 0xD1D1D1AA
#define @GREY82                  0xD1D1D1AA
#define @GREY_82                 0xD1D1D1AA
#define @GRAY83                  0xD4D4D4AA
#define @GRAY_83                 0xD4D4D4AA
#define @GREY83                  0xD4D4D4AA
#define @GREY_83                 0xD4D4D4AA
#define @GRAY84                  0xD6D6D6AA
#define @GRAY_84                 0xD6D6D6AA
#define @GREY84                  0xD6D6D6AA
#define @GREY_84                 0xD6D6D6AA
#define @GRAY85                  0xD9D9D9AA
#define @GRAY_85                 0xD9D9D9AA
#define @GREY85                  0xD9D9D9AA
#define @GREY_85                 0xD9D9D9AA
#define @GRAY86                  0xDBDBDBAA
#define @GRAY_86                 0xDBDBDBAA
#define @GREY86                  0xDBDBDBAA
#define @GREY_86                 0xDBDBDBAA
#define @GRAY87                  0xDEDEDEAA
#define @GRAY_87                 0xDEDEDEAA
#define @GREY87                  0xDEDEDEAA
#define @GREY_87                 0xDEDEDEAA
#define @GRAY88                  0xE0E0E0AA
#define @GRAY_88                 0xE0E0E0AA
#define @GREY88                  0xE0E0E0AA
#define @GREY_88                 0xE0E0E0AA
#define @GRAY89                  0xE3E3E3AA
#define @GRAY_89                 0xE3E3E3AA
#define @GREY89                  0xE3E3E3AA
#define @GREY_89                 0xE3E3E3AA
#define @GRAY90                  0xE5E5E5AA
#define @GRAY_90                 0xE5E5E5AA
#define @GREY90                  0xE5E5E5AA
#define @GREY_90                 0xE5E5E5AA
#define @GRAY91                  0xE8E8E8AA
#define @GRAY_91                 0xE8E8E8AA
#define @GREY91                  0xE8E8E8AA
#define @GREY_91                 0xE8E8E8AA
#define @GRAY92                  0xEBEBEBAA
#define @GRAY_92                 0xEBEBEBAA
#define @GREY92                  0xEBEBEBAA
#define @GREY_92                 0xEBEBEBAA
#define @GRAY93                  0xEDEDEDAA
#define @GRAY_93                 0xEDEDEDAA
#define @GREY93                  0xEDEDEDAA
#define @GREY_93                 0xEDEDEDAA
#define @GRAY94                  0xF0F0F0AA
#define @GRAY_94                 0xF0F0F0AA
#define @GREY94                  0xF0F0F0AA
#define @GREY_94                 0xF0F0F0AA
#define @GRAY95                  0xF2F2F2AA
#define @GRAY_95                 0xF2F2F2AA
#define @GREY95                  0xF2F2F2AA
#define @GREY_95                 0xF2F2F2AA
#define @GRAY96                  0xF5F5F5AA
#define @GRAY_96                 0xF5F5F5AA
#define @GREY96                  0xF5F5F5AA
#define @GREY_96                 0xF5F5F5AA
#define @GRAY97                  0xF7F7F7AA
#define @GRAY_97                 0xF7F7F7AA
#define @GREY97                  0xF7F7F7AA
#define @GREY_97                 0xF7F7F7AA
#define @GRAY98                  0xFAFAFAAA
#define @GRAY_98                 0xFAFAFAAA
#define @GREY98                  0xFAFAFAAA
#define @GREY_98                 0xFAFAFAAA
#define @GRAY99                  0xFCFCFCAA
#define @GRAY_99                 0xFCFCFCAA
#define @GREY99                  0xFCFCFCAA
#define @GREY_99                 0xFCFCFCAA
#define @GRAY100                 0xFFFFFFAA
#define @GRAY_100                0xFFFFFFAA
#define @GREY100                 0xFFFFFFAA
#define @GREY_100                0xFFFFFFAA
#define @DARKGREY                0xA9A9A9AA
#define @DARK_GREY               0xA9A9A9AA
#define @DARKGRAY                0xA9A9A9AA
#define @DARK_GRAY               0xA9A9A9AA
#define @DARKBLUE                0x00008BAA
#define @DARK_BLUE               0x00008BAA
#define @DARKCYAN                0x008B8BAA
#define @DARK_CYAN               0x008B8BAA
#define @DARKMAGENTA             0x8B008BAA
#define @DARK_MAGENTA            0x8B008BAA
#define @DARKRED                 0x8B0000AA
#define @DARK_RED                0x8B0000AA
#define @LIGHTGREEN              0x90EE90AA
#define @LIGHT_GREEN             0x90EE90AA

// Embeddable colours.
#define SNOW                        {FFFAFA}
#define GHOSTWHITE                  {F8F8FF}
#define GHOST_WHITE                 {F8F8FF}
#define WHITESMOKE                  {F5F5F5}
#define WHITE_SMOKE                 {F5F5F5}
#define GAINSBORO                   {DCDCDC}
#define FLORALWHITE                 {FFFAF0}
#define FLORAL_WHITE                {FFFAF0}
#define OLDLACE                     {FDF5E6}
#define OLD_LACE                    {FDF5E6}
#define LINEN                       {FAF0E6}
#define ANTIQUEWHITE                {FAEBD7}
#define ANTIQUE_WHITE               {FAEBD7}
#define PAPAYAWHIP                  {FFEFD5}
#define PAPAYA_WHIP                 {FFEFD5}
#define BLANCHEDALMOND              {FFEBCD}
#define BLANCHED_ALMOND             {FFEBCD}
#define BISQUE                      {FFE4C4}
#define PEACHPUFF                   {FFDAB9}
#define PEACH_PUFF                  {FFDAB9}
#define NAVAJOWHITE                 {FFDEAD}
#define NAVAJO_WHITE                {FFDEAD}
#define MOCCASIN                    {FFE4B5}
#define CORNSILK                    {FFF8DC}
#define IVORY                       {FFFFF0}
#define LEMONCHIFFON                {FFFACD}
#define LEMON_CHIFFON               {FFFACD}
#define SEASHELL                    {FFF5EE}
#define HONEYDEW                    {F0FFF0}
#define MINTCREAM                   {F5FFFA}
#define MINT_CREAM                  {F5FFFA}
#define AZURE                       {F0FFFF}
#define ALICEBLUE                   {F0F8FF}
#define ALICE_BLUE                  {F0F8FF}
#define LAVENDER                    {E6E6FA}
#define LAVENDERBLUSH               {FFF0F5}
#define LAVENDER_BLUSH              {FFF0F5}
#define MISTYROSE                   {FFE4E1}
#define MISTY_ROSE                  {FFE4E1}
#define WHITE                       {FFFFFF}
#define BLACK                       {000000}
#define DARKSLATEGRAY               {2F4F4F}
#define DARK_SLATE_GRAY             {2F4F4F}
#define DARKSLATEGREY               {2F4F4F}
#define DARK_SLATE_GREY             {2F4F4F}
#define DIMGRAY                     {696969}
#define DIM_GRAY                    {696969}
#define DIMGREY                     {696969}
#define DIM_GREY                    {696969}
#define SLATEGRAY                   {708090}
#define SLATE_GRAY                  {708090}
#define SLATEGREY                   {708090}
#define SLATE_GREY                  {708090}
#define LIGHTSLATEGRAY              {778899}
#define LIGHT_SLATE_GRAY            {778899}
#define LIGHTSLATEGREY              {778899}
#define LIGHT_SLATE_GREY            {778899}
#define GRAY                        {BEBEBE}
#define GREY                        {BEBEBE}
#define LIGHTGREY                   {D3D3D3}
#define LIGHT_GREY                  {D3D3D3}
#define LIGHTGRAY                   {D3D3D3}
#define LIGHT_GRAY                  {D3D3D3}
#define MIDNIGHTBLUE                {191970}
#define MIDNIGHT_BLUE               {191970}
#define NAVY                        {000080}
#define NAVYBLUE                    {000080}
#define NAVY_BLUE                   {000080}
#define CORNFLOWERBLUE              {6495ED}
#define CORNFLOWER_BLUE             {6495ED}
#define DARKSLATEBLUE               {483D8B}
#define DARK_SLATE_BLUE             {483D8B}
#define SLATEBLUE                   {6A5ACD}
#define SLATE_BLUE                  {6A5ACD}
#define MEDIUMSLATEBLUE             {7B68EE}
#define MEDIUM_SLATE_BLUE           {7B68EE}
#define LIGHTSLATEBLUE              {8470FF}
#define LIGHT_SLATE_BLUE            {8470FF}
#define MEDIUMBLUE                  {0000CD}
#define MEDIUM_BLUE                 {0000CD}
#define ROYALBLUE                   {4169E1}
#define ROYAL_BLUE                  {4169E1}
#define BLUE                        {0000FF}
#define DODGERBLUE                  {1E90FF}
#define DODGER_BLUE                 {1E90FF}
#define DEEPSKYBLUE                 {00BFFF}
#define DEEP_SKY_BLUE               {00BFFF}
#define SKYBLUE                     {87CEEB}
#define SKY_BLUE                    {87CEEB}
#define LIGHTSKYBLUE                {87CEFA}
#define LIGHT_SKY_BLUE              {87CEFA}
#define STEELBLUE                   {4682B4}
#define STEEL_BLUE                  {4682B4}
#define LIGHTSTEELBLUE              {B0C4DE}
#define LIGHT_STEEL_BLUE            {B0C4DE}
#define LIGHTBLUE                   {ADD8E6}
#define LIGHT_BLUE                  {ADD8E6}
#define POWDERBLUE                  {B0E0E6}
#define POWDER_BLUE                 {B0E0E6}
#define PALETURQUOISE               {AFEEEE}
#define PALE_TURQUOISE              {AFEEEE}
#define DARKTURQUOISE               {00CED1}
#define DARK_TURQUOISE              {00CED1}
#define MEDIUMTURQUOISE             {48D1CC}
#define MEDIUM_TURQUOISE            {48D1CC}
#define TURQUOISE                   {40E0D0}
#define CYAN                        {00FFFF}
#define LIGHTCYAN                   {E0FFFF}
#define LIGHT_CYAN                  {E0FFFF}
#define CADETBLUE                   {5F9EA0}
#define CADET_BLUE                  {5F9EA0}
#define MEDIUMAQUAMARINE            {66CDAA}
#define MEDIUM_AQUAMARINE           {66CDAA}
#define AQUAMARINE                  {7FFFD4}
#define DARKGREEN                   {006400}
#define DARK_GREEN                  {006400}
#define DARKOLIVEGREEN              {556B2F}
#define DARK_OLIVE_GREEN            {556B2F}
#define DARKSEAGREEN                {8FBC8F}
#define DARK_SEA_GREEN              {8FBC8F}
#define SEAGREEN                    {2E8B57}
#define SEA_GREEN                   {2E8B57}
#define MEDIUMSEAGREEN              {3CB371}
#define MEDIUM_SEA_GREEN            {3CB371}
#define LIGHTSEAGREEN               {20B2AA}
#define LIGHT_SEA_GREEN             {20B2AA}
#define PALEGREEN                   {98FB98}
#define PALE_GREEN                  {98FB98}
#define SPRINGGREEN                 {00FF7F}
#define SPRING_GREEN                {00FF7F}
#define LAWNGREEN                   {7CFC00}
#define LAWN_GREEN                  {7CFC00}
#define GREEN                       {00FF00}
#define CHARTREUSE                  {7FFF00}
#define MEDIUMSPRINGGREEN           {00FA9A}
#define MEDIUM_SPRING_GREEN         {00FA9A}
#define GREENYELLOW                 {ADFF2F}
#define GREEN_YELLOW                {ADFF2F}
#define LIMEGREEN                   {32CD32}
#define LIME_GREEN                  {32CD32}
#define YELLOWGREEN                 {9ACD32}
#define YELLOW_GREEN                {9ACD32}
#define FORESTGREEN                 {228B22}
#define FOREST_GREEN                {228B22}
#define OLIVEDRAB                   {6B8E23}
#define OLIVE_DRAB                  {6B8E23}
#define DARKKHAKI                   {BDB76B}
#define DARK_KHAKI                  {BDB76B}
#define KHAKI                       {F0E68C}
#define PALEGOLDENROD               {EEE8AA}
#define PALE_GOLDENROD              {EEE8AA}
#define LIGHTGOLDENRODYELLOW        {FAFAD2}
#define LIGHT_GOLDENROD_YELLOW      {FAFAD2}
#define LIGHTYELLOW                 {FFFFE0}
#define LIGHT_YELLOW                {FFFFE0}
#define YELLOW                      {FFFF00}
#define GOLD                        {FFD700}
#define LIGHTGOLDENROD              {EEDD82}
#define LIGHT_GOLDENROD             {EEDD82}
#define GOLDENROD                   {DAA520}
#define DARKGOLDENROD               {B8860B}
#define DARK_GOLDENROD              {B8860B}
#define ROSYBROWN                   {BC8F8F}
#define ROSY_BROWN                  {BC8F8F}
#define INDIANRED                   {CD5C5C}
#define INDIAN_RED                  {CD5C5C}
#define SADDLEBROWN                 {8B4513}
#define SADDLE_BROWN                {8B4513}
#define SIENNA                      {A0522D}
#define PERU                        {CD853F}
#define BURLYWOOD                   {DEB887}
#define BEIGE                       {F5F5DC}
#define WHEAT                       {F5DEB3}
#define SANDYBROWN                  {F4A460}
#define SANDY_BROWN                 {F4A460}
#define TAN                         {D2B48C}
#define CHOCOLATE                   {D2691E}
#define FIREBRICK                   {B22222}
#define BROWN                       {A52A2A}
#define DARKSALMON                  {E9967A}
#define DARK_SALMON                 {E9967A}
#define SALMON                      {FA8072}
#define LIGHTSALMON                 {FFA07A}
#define LIGHT_SALMON                {FFA07A}
#define ORANGE                      {FFA500}
#define DARKORANGE                  {FF8C00}
#define DARK_ORANGE                 {FF8C00}
#define CORAL                       {FF7F50}
#define LIGHTCORAL                  {F08080}
#define LIGHT_CORAL                 {F08080}
#define TOMATO                      {FF6347}
#define ORANGERED                   {FF4500}
#define ORANGE_RED                  {FF4500}
#define RED                         {FF0000}
#define HOTPINK                     {FF69B4}
#define HOT_PINK                    {FF69B4}
#define DEEPPINK                    {FF1493}
#define DEEP_PINK                   {FF1493}
#define PINK                        {FFC0CB}
#define LIGHTPINK                   {FFB6C1}
#define LIGHT_PINK                  {FFB6C1}
#define PALEVIOLETRED               {DB7093}
#define PALE_VIOLET_RED             {DB7093}
#define MAROON                      {B03060}
#define MEDIUMVIOLETRED             {C71585}
#define MEDIUM_VIOLET_RED           {C71585}
#define VIOLETRED                   {D02090}
#define VIOLET_RED                  {D02090}
#define MAGENTA                     {FF00FF}
#define VIOLET                      {EE82EE}
#define PLUM                        {DDA0DD}
#define ORCHID                      {DA70D6}
#define MEDIUMORCHID                {BA55D3}
#define MEDIUM_ORCHID               {BA55D3}
#define DARKORCHID                  {9932CC}
#define DARK_ORCHID                 {9932CC}
#define DARKVIOLET                  {9400D3}
#define DARK_VIOLET                 {9400D3}
#define BLUEVIOLET                  {8A2BE2}
#define BLUE_VIOLET                 {8A2BE2}
#define PURPLE                      {A020F0}
#define MEDIUMPURPLE                {9370DB}
#define MEDIUM_PURPLE               {9370DB}
#define THISTLE                     {D8BFD8}
#define SNOW1                       {FFFAFA}
#define SNOW_1                      {FFFAFA}
#define SNOW2                       {EEE9E9}
#define SNOW_2                      {EEE9E9}
#define SNOW3                       {CDC9C9}
#define SNOW_3                      {CDC9C9}
#define SNOW4                       {8B8989}
#define SNOW_4                      {8B8989}
#define SEASHELL1                   {FFF5EE}
#define SEASHELL_1                  {FFF5EE}
#define SEASHELL2                   {EEE5DE}
#define SEASHELL_2                  {EEE5DE}
#define SEASHELL3                   {CDC5BF}
#define SEASHELL_3                  {CDC5BF}
#define SEASHELL4                   {8B8682}
#define SEASHELL_4                  {8B8682}
#define ANTIQUEWHITE1               {FFEFDB}
#define ANTIQUE_WHITE_1             {FFEFDB}
#define ANTIQUEWHITE2               {EEDFCC}
#define ANTIQUE_WHITE_2             {EEDFCC}
#define ANTIQUEWHITE3               {CDC0B0}
#define ANTIQUE_WHITE_3             {CDC0B0}
#define ANTIQUEWHITE4               {8B8378}
#define ANTIQUE_WHITE_4             {8B8378}
#define BISQUE1                     {FFE4C4}
#define BISQUE_1                    {FFE4C4}
#define BISQUE2                     {EED5B7}
#define BISQUE_2                    {EED5B7}
#define BISQUE3                     {CDB79E}
#define BISQUE_3                    {CDB79E}
#define BISQUE4                     {8B7D6B}
#define BISQUE_4                    {8B7D6B}
#define PEACHPUFF1                  {FFDAB9}
#define PEACH_PUFF_1                {FFDAB9}
#define PEACHPUFF2                  {EECBAD}
#define PEACH_PUFF_2                {EECBAD}
#define PEACHPUFF3                  {CDAF95}
#define PEACH_PUFF_3                {CDAF95}
#define PEACHPUFF4                  {8B7765}
#define PEACH_PUFF_4                {8B7765}
#define NAVAJOWHITE1                {FFDEAD}
#define NAVAJO_WHITE_1              {FFDEAD}
#define NAVAJOWHITE2                {EECFA1}
#define NAVAJO_WHITE_2              {EECFA1}
#define NAVAJOWHITE3                {CDB38B}
#define NAVAJO_WHITE_3              {CDB38B}
#define NAVAJOWHITE4                {8B795E}
#define NAVAJO_WHITE_4              {8B795E}
#define LEMONCHIFFON1               {FFFACD}
#define LEMON_CHIFFON_1             {FFFACD}
#define LEMONCHIFFON2               {EEE9BF}
#define LEMON_CHIFFON_2             {EEE9BF}
#define LEMONCHIFFON3               {CDC9A5}
#define LEMON_CHIFFON_3             {CDC9A5}
#define LEMONCHIFFON4               {8B8970}
#define LEMON_CHIFFON_4             {8B8970}
#define CORNSILK1                   {FFF8DC}
#define CORNSILK_1                  {FFF8DC}
#define CORNSILK2                   {EEE8CD}
#define CORNSILK_2                  {EEE8CD}
#define CORNSILK3                   {CDC8B1}
#define CORNSILK_3                  {CDC8B1}
#define CORNSILK4                   {8B8878}
#define CORNSILK_4                  {8B8878}
#define IVORY1                      {FFFFF0}
#define IVORY_1                     {FFFFF0}
#define IVORY2                      {EEEEE0}
#define IVORY_2                     {EEEEE0}
#define IVORY3                      {CDCDC1}
#define IVORY_3                     {CDCDC1}
#define IVORY4                      {8B8B83}
#define IVORY_4                     {8B8B83}
#define HONEYDEW1                   {F0FFF0}
#define HONEYDEW_1                  {F0FFF0}
#define HONEYDEW2                   {E0EEE0}
#define HONEYDEW_2                  {E0EEE0}
#define HONEYDEW3                   {C1CDC1}
#define HONEYDEW_3                  {C1CDC1}
#define HONEYDEW4                   {838B83}
#define HONEYDEW_4                  {838B83}
#define LAVENDERBLUSH1              {FFF0F5}
#define LAVENDER_BLUSH_1            {FFF0F5}
#define LAVENDERBLUSH2              {EEE0E5}
#define LAVENDER_BLUSH_2            {EEE0E5}
#define LAVENDERBLUSH3              {CDC1C5}
#define LAVENDER_BLUSH_3            {CDC1C5}
#define LAVENDERBLUSH4              {8B8386}
#define LAVENDER_BLUSH_4            {8B8386}
#define MISTYROSE1                  {FFE4E1}
#define MISTY_ROSE_1                {FFE4E1}
#define MISTYROSE2                  {EED5D2}
#define MISTY_ROSE_2                {EED5D2}
#define MISTYROSE3                  {CDB7B5}
#define MISTY_ROSE_3                {CDB7B5}
#define MISTYROSE4                  {8B7D7B}
#define MISTY_ROSE_4                {8B7D7B}
#define AZURE1                      {F0FFFF}
#define AZURE_1                     {F0FFFF}
#define AZURE2                      {E0EEEE}
#define AZURE_2                     {E0EEEE}
#define AZURE3                      {C1CDCD}
#define AZURE_3                     {C1CDCD}
#define AZURE4                      {838B8B}
#define AZURE_4                     {838B8B}
#define SLATEBLUE1                  {836FFF}
#define SLATE_BLUE_1                {836FFF}
#define SLATEBLUE2                  {7A67EE}
#define SLATE_BLUE_2                {7A67EE}
#define SLATEBLUE3                  {6959CD}
#define SLATE_BLUE_3                {6959CD}
#define SLATEBLUE4                  {473C8B}
#define SLATE_BLUE_4                {473C8B}
#define ROYALBLUE1                  {4876FF}
#define ROYAL_BLUE_1                {4876FF}
#define ROYALBLUE2                  {436EEE}
#define ROYAL_BLUE_2                {436EEE}
#define ROYALBLUE3                  {3A5FCD}
#define ROYAL_BLUE_3                {3A5FCD}
#define ROYALBLUE4                  {27408B}
#define ROYAL_BLUE_4                {27408B}
#define BLUE1                       {0000FF}
#define BLUE_1                      {0000FF}
#define BLUE2                       {0000EE}
#define BLUE_2                      {0000EE}
#define BLUE3                       {0000CD}
#define BLUE_3                      {0000CD}
#define BLUE4                       {00008B}
#define BLUE_4                      {00008B}
#define DODGERBLUE1                 {1E90FF}
#define DODGER_BLUE_1               {1E90FF}
#define DODGERBLUE2                 {1C86EE}
#define DODGER_BLUE_2               {1C86EE}
#define DODGERBLUE3                 {1874CD}
#define DODGER_BLUE_3               {1874CD}
#define DODGERBLUE4                 {104E8B}
#define DODGER_BLUE_4               {104E8B}
#define STEELBLUE1                  {63B8FF}
#define STEEL_BLUE_1                {63B8FF}
#define STEELBLUE2                  {5CACEE}
#define STEEL_BLUE_2                {5CACEE}
#define STEELBLUE3                  {4F94CD}
#define STEEL_BLUE_3                {4F94CD}
#define STEELBLUE4                  {36648B}
#define STEEL_BLUE_4                {36648B}
#define DEEPSKYBLUE1                {00BFFF}
#define DEEP_SKY_BLUE_1             {00BFFF}
#define DEEPSKYBLUE2                {00B2EE}
#define DEEP_SKY_BLUE_2             {00B2EE}
#define DEEPSKYBLUE3                {009ACD}
#define DEEP_SKY_BLUE_3             {009ACD}
#define DEEPSKYBLUE4                {00688B}
#define DEEP_SKY_BLUE_4             {00688B}
#define SKYBLUE1                    {87CEFF}
#define SKY_BLUE_1                  {87CEFF}
#define SKYBLUE2                    {7EC0EE}
#define SKY_BLUE_2                  {7EC0EE}
#define SKYBLUE3                    {6CA6CD}
#define SKY_BLUE_3                  {6CA6CD}
#define SKYBLUE4                    {4A708B}
#define SKY_BLUE_4                  {4A708B}
#define LIGHTSKYBLUE1               {B0E2FF}
#define LIGHT_SKY_BLUE_1            {B0E2FF}
#define LIGHTSKYBLUE2               {A4D3EE}
#define LIGHT_SKY_BLUE_2            {A4D3EE}
#define LIGHTSKYBLUE3               {8DB6CD}
#define LIGHT_SKY_BLUE_3            {8DB6CD}
#define LIGHTSKYBLUE4               {607B8B}
#define LIGHT_SKY_BLUE_4            {607B8B}
#define SLATEGRAY1                  {C6E2FF}
#define SLATE_GRAY_1                {C6E2FF}
#define SLATEGRAY2                  {B9D3EE}
#define SLATE_GRAY_2                {B9D3EE}
#define SLATEGRAY3                  {9FB6CD}
#define SLATE_GRAY_3                {9FB6CD}
#define SLATEGRAY4                  {6C7B8B}
#define SLATE_GRAY_4                {6C7B8B}
#define SLATEGREY1                  {C6E2FF}
#define SLATE_GREY_1                {C6E2FF}
#define SLATEGREY2                  {B9D3EE}
#define SLATE_GREY_2                {B9D3EE}
#define SLATEGREY3                  {9FB6CD}
#define SLATE_GREY_3                {9FB6CD}
#define SLATEGREY4                  {6C7B8B}
#define SLATE_GREY_4                {6C7B8B}
#define LIGHTSTEELBLUE1             {CAE1FF}
#define LIGHT_STEEL_BLUE_1          {CAE1FF}
#define LIGHTSTEELBLUE2             {BCD2EE}
#define LIGHT_STEEL_BLUE_2          {BCD2EE}
#define LIGHTSTEELBLUE3             {A2B5CD}
#define LIGHT_STEEL_BLUE_3          {A2B5CD}
#define LIGHTSTEELBLUE4             {6E7B8B}
#define LIGHT_STEEL_BLUE_4          {6E7B8B}
#define LIGHTBLUE1                  {BFEFFF}
#define LIGHT_BLUE_1                {BFEFFF}
#define LIGHTBLUE2                  {B2DFEE}
#define LIGHT_BLUE_2                {B2DFEE}
#define LIGHTBLUE3                  {9AC0CD}
#define LIGHT_BLUE_3                {9AC0CD}
#define LIGHTBLUE4                  {68838B}
#define LIGHT_BLUE_4                {68838B}
#define LIGHTCYAN1                  {E0FFFF}
#define LIGHT_CYAN_1                {E0FFFF}
#define LIGHTCYAN2                  {D1EEEE}
#define LIGHT_CYAN_2                {D1EEEE}
#define LIGHTCYAN3                  {B4CDCD}
#define LIGHT_CYAN_3                {B4CDCD}
#define LIGHTCYAN4                  {7A8B8B}
#define LIGHT_CYAN_4                {7A8B8B}
#define PALETURQUOISE1              {BBFFFF}
#define PALE_TURQUOISE_1            {BBFFFF}
#define PALETURQUOISE2              {AEEEEE}
#define PALE_TURQUOISE_2            {AEEEEE}
#define PALETURQUOISE3              {96CDCD}
#define PALE_TURQUOISE_3            {96CDCD}
#define PALETURQUOISE4              {668B8B}
#define PALE_TURQUOISE_4            {668B8B}
#define CADETBLUE1                  {98F5FF}
#define CADET_BLUE_1                {98F5FF}
#define CADETBLUE2                  {8EE5EE}
#define CADET_BLUE_2                {8EE5EE}
#define CADETBLUE3                  {7AC5CD}
#define CADET_BLUE_3                {7AC5CD}
#define CADETBLUE4                  {53868B}
#define CADET_BLUE_4                {53868B}
#define TURQUOISE1                  {00F5FF}
#define TURQUOISE_1                 {00F5FF}
#define TURQUOISE2                  {00E5EE}
#define TURQUOISE_2                 {00E5EE}
#define TURQUOISE3                  {00C5CD}
#define TURQUOISE_3                 {00C5CD}
#define TURQUOISE4                  {00868B}
#define TURQUOISE_4                 {00868B}
#define CYAN1                       {00FFFF}
#define CYAN_1                      {00FFFF}
#define CYAN2                       {00EEEE}
#define CYAN_2                      {00EEEE}
#define CYAN3                       {00CDCD}
#define CYAN_3                      {00CDCD}
#define CYAN4                       {008B8B}
#define CYAN_4                      {008B8B}
#define DARKSLATEGRAY1              {97FFFF}
#define DARK_SLATE_GRAY_1           {97FFFF}
#define DARKSLATEGRAY2              {8DEEEE}
#define DARK_SLATE_GRAY_2           {8DEEEE}
#define DARKSLATEGRAY3              {79CDCD}
#define DARK_SLATE_GRAY_3           {79CDCD}
#define DARKSLATEGRAY4              {528B8B}
#define DARK_SLATE_GRAY_4           {528B8B}
#define DARKSLATEGREY1              {97FFFF}
#define DARK_SLATE_GREY_1           {97FFFF}
#define DARKSLATEGREY2              {8DEEEE}
#define DARK_SLATE_GREY_2           {8DEEEE}
#define DARKSLATEGREY3              {79CDCD}
#define DARK_SLATE_GREY_3           {79CDCD}
#define DARKSLATEGREY4              {528B8B}
#define DARK_SLATE_GREY_4           {528B8B}
#define AQUAMARINE1                 {7FFFD4}
#define AQUAMARINE_1                {7FFFD4}
#define AQUAMARINE2                 {76EEC6}
#define AQUAMARINE_2                {76EEC6}
#define AQUAMARINE3                 {66CDAA}
#define AQUAMARINE_3                {66CDAA}
#define AQUAMARINE4                 {458B74}
#define AQUAMARINE_4                {458B74}
#define DARKSEAGREEN1               {C1FFC1}
#define DARK_SEA_GREEN_1            {C1FFC1}
#define DARKSEAGREEN2               {B4EEB4}
#define DARK_SEA_GREEN_2            {B4EEB4}
#define DARKSEAGREEN3               {9BCD9B}
#define DARK_SEA_GREEN_3            {9BCD9B}
#define DARKSEAGREEN4               {698B69}
#define DARK_SEA_GREEN_4            {698B69}
#define SEAGREEN1                   {54FF9F}
#define SEA_GREEN_1                 {54FF9F}
#define SEAGREEN2                   {4EEE94}
#define SEA_GREEN_2                 {4EEE94}
#define SEAGREEN3                   {43CD80}
#define SEA_GREEN_3                 {43CD80}
#define SEAGREEN4                   {2E8B57}
#define SEA_GREEN_4                 {2E8B57}
#define PALEGREEN1                  {9AFF9A}
#define PALE_GREEN_1                {9AFF9A}
#define PALEGREEN2                  {90EE90}
#define PALE_GREEN_2                {90EE90}
#define PALEGREEN3                  {7CCD7C}
#define PALE_GREEN_3                {7CCD7C}
#define PALEGREEN4                  {548B54}
#define PALE_GREEN_4                {548B54}
#define SPRINGGREEN1                {00FF7F}
#define SPRING_GREEN_1              {00FF7F}
#define SPRINGGREEN2                {00EE76}
#define SPRING_GREEN_2              {00EE76}
#define SPRINGGREEN3                {00CD66}
#define SPRING_GREEN_3              {00CD66}
#define SPRINGGREEN4                {008B45}
#define SPRING_GREEN_4              {008B45}
#define GREEN1                      {00FF00}
#define GREEN_1                     {00FF00}
#define GREEN2                      {00EE00}
#define GREEN_2                     {00EE00}
#define GREEN3                      {00CD00}
#define GREEN_3                     {00CD00}
#define GREEN4                      {008B00}
#define GREEN_4                     {008B00}
#define CHARTREUSE1                 {7FFF00}
#define CHARTREUSE_1                {7FFF00}
#define CHARTREUSE2                 {76EE00}
#define CHARTREUSE_2                {76EE00}
#define CHARTREUSE3                 {66CD00}
#define CHARTREUSE_3                {66CD00}
#define CHARTREUSE4                 {458B00}
#define CHARTREUSE_4                {458B00}
#define OLIVEDRAB1                  {C0FF3E}
#define OLIVE_DRAB_1                {C0FF3E}
#define OLIVEDRAB2                  {B3EE3A}
#define OLIVE_DRAB_2                {B3EE3A}
#define OLIVEDRAB3                  {9ACD32}
#define OLIVE_DRAB_3                {9ACD32}
#define OLIVEDRAB4                  {698B22}
#define OLIVE_DRAB_4                {698B22}
#define DARKOLIVEGREEN1             {CAFF70}
#define DARK_OLIVE_GREEN_1          {CAFF70}
#define DARKOLIVEGREEN2             {BCEE68}
#define DARK_OLIVE_GREEN_2          {BCEE68}
#define DARKOLIVEGREEN3             {A2CD5A}
#define DARK_OLIVE_GREEN_3          {A2CD5A}
#define DARKOLIVEGREEN4             {6E8B3D}
#define DARK_OLIVE_GREEN_4          {6E8B3D}
#define KHAKI1                      {FFF68F}
#define KHAKI_1                     {FFF68F}
#define KHAKI2                      {EEE685}
#define KHAKI_2                     {EEE685}
#define KHAKI3                      {CDC673}
#define KHAKI_3                     {CDC673}
#define KHAKI4                      {8B864E}
#define KHAKI_4                     {8B864E}
#define LIGHTGOLDENROD1             {FFEC8B}
#define LIGHT_GOLDENROD_1           {FFEC8B}
#define LIGHTGOLDENROD2             {EEDC82}
#define LIGHT_GOLDENROD_2           {EEDC82}
#define LIGHTGOLDENROD3             {CDBE70}
#define LIGHT_GOLDENROD_3           {CDBE70}
#define LIGHTGOLDENROD4             {8B814C}
#define LIGHT_GOLDENROD_4           {8B814C}
#define LIGHTYELLOW1                {FFFFE0}
#define LIGHT_YELLOW_1              {FFFFE0}
#define LIGHTYELLOW2                {EEEED1}
#define LIGHT_YELLOW_2              {EEEED1}
#define LIGHTYELLOW3                {CDCDB4}
#define LIGHT_YELLOW_3              {CDCDB4}
#define LIGHTYELLOW4                {8B8B7A}
#define LIGHT_YELLOW_4              {8B8B7A}
#define YELLOW1                     {FFFF00}
#define YELLOW_1                    {FFFF00}
#define YELLOW2                     {EEEE00}
#define YELLOW_2                    {EEEE00}
#define YELLOW3                     {CDCD00}
#define YELLOW_3                    {CDCD00}
#define YELLOW4                     {8B8B00}
#define YELLOW_4                    {8B8B00}
#define GOLD1                       {FFD700}
#define GOLD_1                      {FFD700}
#define GOLD2                       {EEC900}
#define GOLD_2                      {EEC900}
#define GOLD3                       {CDAD00}
#define GOLD_3                      {CDAD00}
#define GOLD4                       {8B7500}
#define GOLD_4                      {8B7500}
#define GOLDENROD1                  {FFC125}
#define GOLDENROD_1                 {FFC125}
#define GOLDENROD2                  {EEB422}
#define GOLDENROD_2                 {EEB422}
#define GOLDENROD3                  {CD9B1D}
#define GOLDENROD_3                 {CD9B1D}
#define GOLDENROD4                  {8B6914}
#define GOLDENROD_4                 {8B6914}
#define DARKGOLDENROD1              {FFB90F}
#define DARK_GOLDENROD_1            {FFB90F}
#define DARKGOLDENROD2              {EEAD0E}
#define DARK_GOLDENROD_2            {EEAD0E}
#define DARKGOLDENROD3              {CD950C}
#define DARK_GOLDENROD_3            {CD950C}
#define DARKGOLDENROD4              {8B6508}
#define DARK_GOLDENROD_4            {8B6508}
#define ROSYBROWN1                  {FFC1C1}
#define ROSY_BROWN_1                {FFC1C1}
#define ROSYBROWN2                  {EEB4B4}
#define ROSY_BROWN_2                {EEB4B4}
#define ROSYBROWN3                  {CD9B9B}
#define ROSY_BROWN_3                {CD9B9B}
#define ROSYBROWN4                  {8B6969}
#define ROSY_BROWN_4                {8B6969}
#define INDIANRED1                  {FF6A6A}
#define INDIAN_RED_1                {FF6A6A}
#define INDIANRED2                  {EE6363}
#define INDIAN_RED_2                {EE6363}
#define INDIANRED3                  {CD5555}
#define INDIAN_RED_3                {CD5555}
#define INDIANRED4                  {8B3A3A}
#define INDIAN_RED_4                {8B3A3A}
#define SIENNA1                     {FF8247}
#define SIENNA_1                    {FF8247}
#define SIENNA2                     {EE7942}
#define SIENNA_2                    {EE7942}
#define SIENNA3                     {CD6839}
#define SIENNA_3                    {CD6839}
#define SIENNA4                     {8B4726}
#define SIENNA_4                    {8B4726}
#define BURLYWOOD1                  {FFD39B}
#define BURLYWOOD_1                 {FFD39B}
#define BURLYWOOD2                  {EEC591}
#define BURLYWOOD_2                 {EEC591}
#define BURLYWOOD3                  {CDAA7D}
#define BURLYWOOD_3                 {CDAA7D}
#define BURLYWOOD4                  {8B7355}
#define BURLYWOOD_4                 {8B7355}
#define WHEAT1                      {FFE7BA}
#define WHEAT_1                     {FFE7BA}
#define WHEAT2                      {EED8AE}
#define WHEAT_2                     {EED8AE}
#define WHEAT3                      {CDBA96}
#define WHEAT_3                     {CDBA96}
#define WHEAT4                      {8B7E66}
#define WHEAT_4                     {8B7E66}
#define TAN1                        {FFA54F}
#define TAN_1                       {FFA54F}
#define TAN2                        {EE9A49}
#define TAN_2                       {EE9A49}
#define TAN3                        {CD853F}
#define TAN_3                       {CD853F}
#define TAN4                        {8B5A2B}
#define TAN_4                       {8B5A2B}
#define CHOCOLATE1                  {FF7F24}
#define CHOCOLATE_1                 {FF7F24}
#define CHOCOLATE2                  {EE7621}
#define CHOCOLATE_2                 {EE7621}
#define CHOCOLATE3                  {CD661D}
#define CHOCOLATE_3                 {CD661D}
#define CHOCOLATE4                  {8B4513}
#define CHOCOLATE_4                 {8B4513}
#define FIREBRICK1                  {FF3030}
#define FIREBRICK_1                 {FF3030}
#define FIREBRICK2                  {EE2C2C}
#define FIREBRICK_2                 {EE2C2C}
#define FIREBRICK3                  {CD2626}
#define FIREBRICK_3                 {CD2626}
#define FIREBRICK4                  {8B1A1A}
#define FIREBRICK_4                 {8B1A1A}
#define BROWN1                      {FF4040}
#define BROWN_1                     {FF4040}
#define BROWN2                      {EE3B3B}
#define BROWN_2                     {EE3B3B}
#define BROWN3                      {CD3333}
#define BROWN_3                     {CD3333}
#define BROWN4                      {8B2323}
#define BROWN_4                     {8B2323}
#define SALMON1                     {FF8C69}
#define SALMON_1                    {FF8C69}
#define SALMON2                     {EE8262}
#define SALMON_2                    {EE8262}
#define SALMON3                     {CD7054}
#define SALMON_3                    {CD7054}
#define SALMON4                     {8B4C39}
#define SALMON_4                    {8B4C39}
#define LIGHTSALMON1                {FFA07A}
#define LIGHT_SALMON_1              {FFA07A}
#define LIGHTSALMON2                {EE9572}
#define LIGHT_SALMON_2              {EE9572}
#define LIGHTSALMON3                {CD8162}
#define LIGHT_SALMON_3              {CD8162}
#define LIGHTSALMON4                {8B5742}
#define LIGHT_SALMON_4              {8B5742}
#define ORANGE1                     {FFA500}
#define ORANGE_1                    {FFA500}
#define ORANGE2                     {EE9A00}
#define ORANGE_2                    {EE9A00}
#define ORANGE3                     {CD8500}
#define ORANGE_3                    {CD8500}
#define ORANGE4                     {8B5A00}
#define ORANGE_4                    {8B5A00}
#define DARKORANGE1                 {FF7F00}
#define DARK_ORANGE_1               {FF7F00}
#define DARKORANGE2                 {EE7600}
#define DARK_ORANGE_2               {EE7600}
#define DARKORANGE3                 {CD6600}
#define DARK_ORANGE_3               {CD6600}
#define DARKORANGE4                 {8B4500}
#define DARK_ORANGE_4               {8B4500}
#define CORAL1                      {FF7256}
#define CORAL_1                     {FF7256}
#define CORAL2                      {EE6A50}
#define CORAL_2                     {EE6A50}
#define CORAL3                      {CD5B45}
#define CORAL_3                     {CD5B45}
#define CORAL4                      {8B3E2F}
#define CORAL_4                     {8B3E2F}
#define TOMATO1                     {FF6347}
#define TOMATO_1                    {FF6347}
#define TOMATO2                     {EE5C42}
#define TOMATO_2                    {EE5C42}
#define TOMATO3                     {CD4F39}
#define TOMATO_3                    {CD4F39}
#define TOMATO4                     {8B3626}
#define TOMATO_4                    {8B3626}
#define ORANGERED1                  {FF4500}
#define ORANGE_RED_1                {FF4500}
#define ORANGERED2                  {EE4000}
#define ORANGE_RED_2                {EE4000}
#define ORANGERED3                  {CD3700}
#define ORANGE_RED_3                {CD3700}
#define ORANGERED4                  {8B2500}
#define ORANGE_RED_4                {8B2500}
#define RED1                        {FF0000}
#define RED_1                       {FF0000}
#define RED2                        {EE0000}
#define RED_2                       {EE0000}
#define RED3                        {CD0000}
#define RED_3                       {CD0000}
#define RED4                        {8B0000}
#define RED_4                       {8B0000}
#define DEEPPINK1                   {FF1493}
#define DEEP_PINK_1                 {FF1493}
#define DEEPPINK2                   {EE1289}
#define DEEP_PINK_2                 {EE1289}
#define DEEPPINK3                   {CD1076}
#define DEEP_PINK_3                 {CD1076}
#define DEEPPINK4                   {8B0A50}
#define DEEP_PINK_4                 {8B0A50}
#define HOTPINK1                    {FF6EB4}
#define HOT_PINK_1                  {FF6EB4}
#define HOTPINK2                    {EE6AA7}
#define HOT_PINK_2                  {EE6AA7}
#define HOTPINK3                    {CD6090}
#define HOT_PINK_3                  {CD6090}
#define HOTPINK4                    {8B3A62}
#define HOT_PINK_4                  {8B3A62}
#define PINK1                       {FFB5C5}
#define PINK_1                      {FFB5C5}
#define PINK2                       {EEA9B8}
#define PINK_2                      {EEA9B8}
#define PINK3                       {CD919E}
#define PINK_3                      {CD919E}
#define PINK4                       {8B636C}
#define PINK_4                      {8B636C}
#define LIGHTPINK1                  {FFAEB9}
#define LIGHT_PINK_1                {FFAEB9}
#define LIGHTPINK2                  {EEA2AD}
#define LIGHT_PINK_2                {EEA2AD}
#define LIGHTPINK3                  {CD8C95}
#define LIGHT_PINK_3                {CD8C95}
#define LIGHTPINK4                  {8B5F65}
#define LIGHT_PINK_4                {8B5F65}
#define PALEVIOLETRED1              {FF82AB}
#define PALE_VIOLET_RED_1           {FF82AB}
#define PALEVIOLETRED2              {EE799F}
#define PALE_VIOLET_RED_2           {EE799F}
#define PALEVIOLETRED3              {CD6889}
#define PALE_VIOLET_RED_3           {CD6889}
#define PALEVIOLETRED4              {8B475D}
#define PALE_VIOLET_RED_4           {8B475D}
#define MAROON1                     {FF34B3}
#define MAROON_1                    {FF34B3}
#define MAROON2                     {EE30A7}
#define MAROON_2                    {EE30A7}
#define MAROON3                     {CD2990}
#define MAROON_3                    {CD2990}
#define MAROON4                     {8B1C62}
#define MAROON_4                    {8B1C62}
#define VIOLETRED1                  {FF3E96}
#define VIOLET_RED_1                {FF3E96}
#define VIOLETRED2                  {EE3A8C}
#define VIOLET_RED_2                {EE3A8C}
#define VIOLETRED3                  {CD3278}
#define VIOLET_RED_3                {CD3278}
#define VIOLETRED4                  {8B2252}
#define VIOLET_RED_4                {8B2252}
#define MAGENTA1                    {FF00FF}
#define MAGENTA_1                   {FF00FF}
#define MAGENTA2                    {EE00EE}
#define MAGENTA_2                   {EE00EE}
#define MAGENTA3                    {CD00CD}
#define MAGENTA_3                   {CD00CD}
#define MAGENTA4                    {8B008B}
#define MAGENTA_4                   {8B008B}
#define ORCHID1                     {FF83FA}
#define ORCHID_1                    {FF83FA}
#define ORCHID2                     {EE7AE9}
#define ORCHID_2                    {EE7AE9}
#define ORCHID3                     {CD69C9}
#define ORCHID_3                    {CD69C9}
#define ORCHID4                     {8B4789}
#define ORCHID_4                    {8B4789}
#define PLUM1                       {FFBBFF}
#define PLUM_1                      {FFBBFF}
#define PLUM2                       {EEAEEE}
#define PLUM_2                      {EEAEEE}
#define PLUM3                       {CD96CD}
#define PLUM_3                      {CD96CD}
#define PLUM4                       {8B668B}
#define PLUM_4                      {8B668B}
#define MEDIUMORCHID1               {E066FF}
#define MEDIUM_ORCHID_1             {E066FF}
#define MEDIUMORCHID2               {D15FEE}
#define MEDIUM_ORCHID_2             {D15FEE}
#define MEDIUMORCHID3               {B452CD}
#define MEDIUM_ORCHID_3             {B452CD}
#define MEDIUMORCHID4               {7A378B}
#define MEDIUM_ORCHID_4             {7A378B}
#define DARKORCHID1                 {BF3EFF}
#define DARK_ORCHID_1               {BF3EFF}
#define DARKORCHID2                 {B23AEE}
#define DARK_ORCHID_2               {B23AEE}
#define DARKORCHID3                 {9A32CD}
#define DARK_ORCHID_3               {9A32CD}
#define DARKORCHID4                 {68228B}
#define DARK_ORCHID_4               {68228B}
#define PURPLE1                     {9B30FF}
#define PURPLE_1                    {9B30FF}
#define PURPLE2                     {912CEE}
#define PURPLE_2                    {912CEE}
#define PURPLE3                     {7D26CD}
#define PURPLE_3                    {7D26CD}
#define PURPLE4                     {551A8B}
#define PURPLE_4                    {551A8B}
#define MEDIUMPURPLE1               {AB82FF}
#define MEDIUM_PURPLE_1             {AB82FF}
#define MEDIUMPURPLE2               {9F79EE}
#define MEDIUM_PURPLE_2             {9F79EE}
#define MEDIUMPURPLE3               {8968CD}
#define MEDIUM_PURPLE_3             {8968CD}
#define MEDIUMPURPLE4               {5D478B}
#define MEDIUM_PURPLE_4             {5D478B}
#define THISTLE1                    {FFE1FF}
#define THISTLE_1                   {FFE1FF}
#define THISTLE2                    {EED2EE}
#define THISTLE_2                   {EED2EE}
#define THISTLE3                    {CDB5CD}
#define THISTLE_3                   {CDB5CD}
#define THISTLE4                    {8B7B8B}
#define THISTLE_4                   {8B7B8B}
#define GRAY0                       {000000}
#define GRAY_0                      {000000}
#define GREY0                       {000000}
#define GREY_0                      {000000}
#define GRAY1                       {030303}
#define GRAY_1                      {030303}
#define GREY1                       {030303}
#define GREY_1                      {030303}
#define GRAY2                       {050505}
#define GRAY_2                      {050505}
#define GREY2                       {050505}
#define GREY_2                      {050505}
#define GRAY3                       {080808}
#define GRAY_3                      {080808}
#define GREY3                       {080808}
#define GREY_3                      {080808}
#define GRAY4                       {0A0A0A}
#define GRAY_4                      {0A0A0A}
#define GREY4                       {0A0A0A}
#define GREY_4                      {0A0A0A}
#define GRAY5                       {0D0D0D}
#define GRAY_5                      {0D0D0D}
#define GREY5                       {0D0D0D}
#define GREY_5                      {0D0D0D}
#define GRAY6                       {0F0F0F}
#define GRAY_6                      {0F0F0F}
#define GREY6                       {0F0F0F}
#define GREY_6                      {0F0F0F}
#define GRAY7                       {121212}
#define GRAY_7                      {121212}
#define GREY7                       {121212}
#define GREY_7                      {121212}
#define GRAY8                       {141414}
#define GRAY_8                      {141414}
#define GREY8                       {141414}
#define GREY_8                      {141414}
#define GRAY9                       {171717}
#define GRAY_9                      {171717}
#define GREY9                       {171717}
#define GREY_9                      {171717}
#define GRAY10                      {1A1A1A}
#define GRAY_10                     {1A1A1A}
#define GREY10                      {1A1A1A}
#define GREY_10                     {1A1A1A}
#define GRAY11                      {1C1C1C}
#define GRAY_11                     {1C1C1C}
#define GREY11                      {1C1C1C}
#define GREY_11                     {1C1C1C}
#define GRAY12                      {1F1F1F}
#define GRAY_12                     {1F1F1F}
#define GREY12                      {1F1F1F}
#define GREY_12                     {1F1F1F}
#define GRAY13                      {212121}
#define GRAY_13                     {212121}
#define GREY13                      {212121}
#define GREY_13                     {212121}
#define GRAY14                      {242424}
#define GRAY_14                     {242424}
#define GREY14                      {242424}
#define GREY_14                     {242424}
#define GRAY15                      {262626}
#define GRAY_15                     {262626}
#define GREY15                      {262626}
#define GREY_15                     {262626}
#define GRAY16                      {292929}
#define GRAY_16                     {292929}
#define GREY16                      {292929}
#define GREY_16                     {292929}
#define GRAY17                      {2B2B2B}
#define GRAY_17                     {2B2B2B}
#define GREY17                      {2B2B2B}
#define GREY_17                     {2B2B2B}
#define GRAY18                      {2E2E2E}
#define GRAY_18                     {2E2E2E}
#define GREY18                      {2E2E2E}
#define GREY_18                     {2E2E2E}
#define GRAY19                      {303030}
#define GRAY_19                     {303030}
#define GREY19                      {303030}
#define GREY_19                     {303030}
#define GRAY20                      {333333}
#define GRAY_20                     {333333}
#define GREY20                      {333333}
#define GREY_20                     {333333}
#define GRAY21                      {363636}
#define GRAY_21                     {363636}
#define GREY21                      {363636}
#define GREY_21                     {363636}
#define GRAY22                      {383838}
#define GRAY_22                     {383838}
#define GREY22                      {383838}
#define GREY_22                     {383838}
#define GRAY23                      {3B3B3B}
#define GRAY_23                     {3B3B3B}
#define GREY23                      {3B3B3B}
#define GREY_23                     {3B3B3B}
#define GRAY24                      {3D3D3D}
#define GRAY_24                     {3D3D3D}
#define GREY24                      {3D3D3D}
#define GREY_24                     {3D3D3D}
#define GRAY25                      {404040}
#define GRAY_25                     {404040}
#define GREY25                      {404040}
#define GREY_25                     {404040}
#define GRAY26                      {424242}
#define GRAY_26                     {424242}
#define GREY26                      {424242}
#define GREY_26                     {424242}
#define GRAY27                      {454545}
#define GRAY_27                     {454545}
#define GREY27                      {454545}
#define GREY_27                     {454545}
#define GRAY28                      {474747}
#define GRAY_28                     {474747}
#define GREY28                      {474747}
#define GREY_28                     {474747}
#define GRAY29                      {4A4A4A}
#define GRAY_29                     {4A4A4A}
#define GREY29                      {4A4A4A}
#define GREY_29                     {4A4A4A}
#define GRAY30                      {4D4D4D}
#define GRAY_30                     {4D4D4D}
#define GREY30                      {4D4D4D}
#define GREY_30                     {4D4D4D}
#define GRAY31                      {4F4F4F}
#define GRAY_31                     {4F4F4F}
#define GREY31                      {4F4F4F}
#define GREY_31                     {4F4F4F}
#define GRAY32                      {525252}
#define GRAY_32                     {525252}
#define GREY32                      {525252}
#define GREY_32                     {525252}
#define GRAY33                      {545454}
#define GRAY_33                     {545454}
#define GREY33                      {545454}
#define GREY_33                     {545454}
#define GRAY34                      {575757}
#define GRAY_34                     {575757}
#define GREY34                      {575757}
#define GREY_34                     {575757}
#define GRAY35                      {595959}
#define GRAY_35                     {595959}
#define GREY35                      {595959}
#define GREY_35                     {595959}
#define GRAY36                      {5C5C5C}
#define GRAY_36                     {5C5C5C}
#define GREY36                      {5C5C5C}
#define GREY_36                     {5C5C5C}
#define GRAY37                      {5E5E5E}
#define GRAY_37                     {5E5E5E}
#define GREY37                      {5E5E5E}
#define GREY_37                     {5E5E5E}
#define GRAY38                      {616161}
#define GRAY_38                     {616161}
#define GREY38                      {616161}
#define GREY_38                     {616161}
#define GRAY39                      {636363}
#define GRAY_39                     {636363}
#define GREY39                      {636363}
#define GREY_39                     {636363}
#define GRAY40                      {666666}
#define GRAY_40                     {666666}
#define GREY40                      {666666}
#define GREY_40                     {666666}
#define GRAY41                      {696969}
#define GRAY_41                     {696969}
#define GREY41                      {696969}
#define GREY_41                     {696969}
#define GRAY42                      {6B6B6B}
#define GRAY_42                     {6B6B6B}
#define GREY42                      {6B6B6B}
#define GREY_42                     {6B6B6B}
#define GRAY43                      {6E6E6E}
#define GRAY_43                     {6E6E6E}
#define GREY43                      {6E6E6E}
#define GREY_43                     {6E6E6E}
#define GRAY44                      {707070}
#define GRAY_44                     {707070}
#define GREY44                      {707070}
#define GREY_44                     {707070}
#define GRAY45                      {737373}
#define GRAY_45                     {737373}
#define GREY45                      {737373}
#define GREY_45                     {737373}
#define GRAY46                      {757575}
#define GRAY_46                     {757575}
#define GREY46                      {757575}
#define GREY_46                     {757575}
#define GRAY47                      {787878}
#define GRAY_47                     {787878}
#define GREY47                      {787878}
#define GREY_47                     {787878}
#define GRAY48                      {7A7A7A}
#define GRAY_48                     {7A7A7A}
#define GREY48                      {7A7A7A}
#define GREY_48                     {7A7A7A}
#define GRAY49                      {7D7D7D}
#define GRAY_49                     {7D7D7D}
#define GREY49                      {7D7D7D}
#define GREY_49                     {7D7D7D}
#define GRAY50                      {7F7F7F}
#define GRAY_50                     {7F7F7F}
#define GREY50                      {7F7F7F}
#define GREY_50                     {7F7F7F}
#define GRAY51                      {828282}
#define GRAY_51                     {828282}
#define GREY51                      {828282}
#define GREY_51                     {828282}
#define GRAY52                      {858585}
#define GRAY_52                     {858585}
#define GREY52                      {858585}
#define GREY_52                     {858585}
#define GRAY53                      {878787}
#define GRAY_53                     {878787}
#define GREY53                      {878787}
#define GREY_53                     {878787}
#define GRAY54                      {8A8A8A}
#define GRAY_54                     {8A8A8A}
#define GREY54                      {8A8A8A}
#define GREY_54                     {8A8A8A}
#define GRAY55                      {8C8C8C}
#define GRAY_55                     {8C8C8C}
#define GREY55                      {8C8C8C}
#define GREY_55                     {8C8C8C}
#define GRAY56                      {8F8F8F}
#define GRAY_56                     {8F8F8F}
#define GREY56                      {8F8F8F}
#define GREY_56                     {8F8F8F}
#define GRAY57                      {919191}
#define GRAY_57                     {919191}
#define GREY57                      {919191}
#define GREY_57                     {919191}
#define GRAY58                      {949494}
#define GRAY_58                     {949494}
#define GREY58                      {949494}
#define GREY_58                     {949494}
#define GRAY59                      {969696}
#define GRAY_59                     {969696}
#define GREY59                      {969696}
#define GREY_59                     {969696}
#define GRAY60                      {999999}
#define GRAY_60                     {999999}
#define GREY60                      {999999}
#define GREY_60                     {999999}
#define GRAY61                      {9C9C9C}
#define GRAY_61                     {9C9C9C}
#define GREY61                      {9C9C9C}
#define GREY_61                     {9C9C9C}
#define GRAY62                      {9E9E9E}
#define GRAY_62                     {9E9E9E}
#define GREY62                      {9E9E9E}
#define GREY_62                     {9E9E9E}
#define GRAY63                      {A1A1A1}
#define GRAY_63                     {A1A1A1}
#define GREY63                      {A1A1A1}
#define GREY_63                     {A1A1A1}
#define GRAY64                      {A3A3A3}
#define GRAY_64                     {A3A3A3}
#define GREY64                      {A3A3A3}
#define GREY_64                     {A3A3A3}
#define GRAY65                      {A6A6A6}
#define GRAY_65                     {A6A6A6}
#define GREY65                      {A6A6A6}
#define GREY_65                     {A6A6A6}
#define GRAY66                      {A8A8A8}
#define GRAY_66                     {A8A8A8}
#define GREY66                      {A8A8A8}
#define GREY_66                     {A8A8A8}
#define GRAY67                      {ABABAB}
#define GRAY_67                     {ABABAB}
#define GREY67                      {ABABAB}
#define GREY_67                     {ABABAB}
#define GRAY68                      {ADADAD}
#define GRAY_68                     {ADADAD}
#define GREY68                      {ADADAD}
#define GREY_68                     {ADADAD}
#define GRAY69                      {B0B0B0}
#define GRAY_69                     {B0B0B0}
#define GREY69                      {B0B0B0}
#define GREY_69                     {B0B0B0}
#define GRAY70                      {B3B3B3}
#define GRAY_70                     {B3B3B3}
#define GREY70                      {B3B3B3}
#define GREY_70                     {B3B3B3}
#define GRAY71                      {B5B5B5}
#define GRAY_71                     {B5B5B5}
#define GREY71                      {B5B5B5}
#define GREY_71                     {B5B5B5}
#define GRAY72                      {B8B8B8}
#define GRAY_72                     {B8B8B8}
#define GREY72                      {B8B8B8}
#define GREY_72                     {B8B8B8}
#define GRAY73                      {BABABA}
#define GRAY_73                     {BABABA}
#define GREY73                      {BABABA}
#define GREY_73                     {BABABA}
#define GRAY74                      {BDBDBD}
#define GRAY_74                     {BDBDBD}
#define GREY74                      {BDBDBD}
#define GREY_74                     {BDBDBD}
#define GRAY75                      {BFBFBF}
#define GRAY_75                     {BFBFBF}
#define GREY75                      {BFBFBF}
#define GREY_75                     {BFBFBF}
#define GRAY76                      {C2C2C2}
#define GRAY_76                     {C2C2C2}
#define GREY76                      {C2C2C2}
#define GREY_76                     {C2C2C2}
#define GRAY77                      {C4C4C4}
#define GRAY_77                     {C4C4C4}
#define GREY77                      {C4C4C4}
#define GREY_77                     {C4C4C4}
#define GRAY78                      {C7C7C7}
#define GRAY_78                     {C7C7C7}
#define GREY78                      {C7C7C7}
#define GREY_78                     {C7C7C7}
#define GRAY79                      {C9C9C9}
#define GRAY_79                     {C9C9C9}
#define GREY79                      {C9C9C9}
#define GREY_79                     {C9C9C9}
#define GRAY80                      {CCCCCC}
#define GRAY_80                     {CCCCCC}
#define GREY80                      {CCCCCC}
#define GREY_80                     {CCCCCC}
#define GRAY81                      {CFCFCF}
#define GRAY_81                     {CFCFCF}
#define GREY81                      {CFCFCF}
#define GREY_81                     {CFCFCF}
#define GRAY82                      {D1D1D1}
#define GRAY_82                     {D1D1D1}
#define GREY82                      {D1D1D1}
#define GREY_82                     {D1D1D1}
#define GRAY83                      {D4D4D4}
#define GRAY_83                     {D4D4D4}
#define GREY83                      {D4D4D4}
#define GREY_83                     {D4D4D4}
#define GRAY84                      {D6D6D6}
#define GRAY_84                     {D6D6D6}
#define GREY84                      {D6D6D6}
#define GREY_84                     {D6D6D6}
#define GRAY85                      {D9D9D9}
#define GRAY_85                     {D9D9D9}
#define GREY85                      {D9D9D9}
#define GREY_85                     {D9D9D9}
#define GRAY86                      {DBDBDB}
#define GRAY_86                     {DBDBDB}
#define GREY86                      {DBDBDB}
#define GREY_86                     {DBDBDB}
#define GRAY87                      {DEDEDE}
#define GRAY_87                     {DEDEDE}
#define GREY87                      {DEDEDE}
#define GREY_87                     {DEDEDE}
#define GRAY88                      {E0E0E0}
#define GRAY_88                     {E0E0E0}
#define GREY88                      {E0E0E0}
#define GREY_88                     {E0E0E0}
#define GRAY89                      {E3E3E3}
#define GRAY_89                     {E3E3E3}
#define GREY89                      {E3E3E3}
#define GREY_89                     {E3E3E3}
#define GRAY90                      {E5E5E5}
#define GRAY_90                     {E5E5E5}
#define GREY90                      {E5E5E5}
#define GREY_90                     {E5E5E5}
#define GRAY91                      {E8E8E8}
#define GRAY_91                     {E8E8E8}
#define GREY91                      {E8E8E8}
#define GREY_91                     {E8E8E8}
#define GRAY92                      {EBEBEB}
#define GRAY_92                     {EBEBEB}
#define GREY92                      {EBEBEB}
#define GREY_92                     {EBEBEB}
#define GRAY93                      {EDEDED}
#define GRAY_93                     {EDEDED}
#define GREY93                      {EDEDED}
#define GREY_93                     {EDEDED}
#define GRAY94                      {F0F0F0}
#define GRAY_94                     {F0F0F0}
#define GREY94                      {F0F0F0}
#define GREY_94                     {F0F0F0}
#define GRAY95                      {F2F2F2}
#define GRAY_95                     {F2F2F2}
#define GREY95                      {F2F2F2}
#define GREY_95                     {F2F2F2}
#define GRAY96                      {F5F5F5}
#define GRAY_96                     {F5F5F5}
#define GREY96                      {F5F5F5}
#define GREY_96                     {F5F5F5}
#define GRAY97                      {F7F7F7}
#define GRAY_97                     {F7F7F7}
#define GREY97                      {F7F7F7}
#define GREY_97                     {F7F7F7}
#define GRAY98                      {FAFAFA}
#define GRAY_98                     {FAFAFA}
#define GREY98                      {FAFAFA}
#define GREY_98                     {FAFAFA}
#define GRAY99                      {FCFCFC}
#define GRAY_99                     {FCFCFC}
#define GREY99                      {FCFCFC}
#define GREY_99                     {FCFCFC}
#define GRAY100                     {FFFFFF}
#define GRAY_100                    {FFFFFF}
#define GREY100                     {FFFFFF}
#define GREY_100                    {FFFFFF}
#define DARKGREY                    {A9A9A9}
#define DARK_GREY                   {A9A9A9}
#define DARKGRAY                    {A9A9A9}
#define DARK_GRAY                   {A9A9A9}
#define DARKBLUE                    {00008B}
#define DARK_BLUE                   {00008B}
#define DARKCYAN                    {008B8B}
#define DARK_CYAN                   {008B8B}
#define DARKMAGENTA                 {8B008B}
#define DARK_MAGENTA                {8B008B}
#define DARKRED                     {8B0000}
#define DARK_RED                    {8B0000}
#define LIGHTGREEN                  {90EE90}
#define LIGHT_GREEN                 {90EE90}

// Colours without alpha.
#define _SNOW                      0xFFFAFA00
#define _GHOSTWHITE                0xF8F8FF00
#define _GHOST_WHITE               0xF8F8FF00
#define _WHITESMOKE                0xF5F5F500
#define _WHITE_SMOKE               0xF5F5F500
#define _GAINSBORO                 0xDCDCDC00
#define _FLORALWHITE               0xFFFAF000
#define _FLORAL_WHITE              0xFFFAF000
#define _OLDLACE                   0xFDF5E600
#define _OLD_LACE                  0xFDF5E600
#define _LINEN                     0xFAF0E600
#define _ANTIQUEWHITE              0xFAEBD700
#define _ANTIQUE_WHITE             0xFAEBD700
#define _PAPAYAWHIP                0xFFEFD500
#define _PAPAYA_WHIP               0xFFEFD500
#define _BLANCHEDALMOND            0xFFEBCD00
#define _BLANCHED_ALMOND           0xFFEBCD00
#define _BISQUE                    0xFFE4C400
#define _PEACHPUFF                 0xFFDAB900
#define _PEACH_PUFF                0xFFDAB900
#define _NAVAJOWHITE               0xFFDEAD00
#define _NAVAJO_WHITE              0xFFDEAD00
#define _MOCCASIN                  0xFFE4B500
#define _CORNSILK                  0xFFF8DC00
#define _IVORY                     0xFFFFF000
#define _LEMONCHIFFON              0xFFFACD00
#define _LEMON_CHIFFON             0xFFFACD00
#define _SEASHELL                  0xFFF5EE00
#define _HONEYDEW                  0xF0FFF000
#define _MINTCREAM                 0xF5FFFA00
#define _MINT_CREAM                0xF5FFFA00
#define _AZURE                     0xF0FFFF00
#define _ALICEBLUE                 0xF0F8FF00
#define _ALICE_BLUE                0xF0F8FF00
#define _LAVENDER                  0xE6E6FA00
#define _LAVENDERBLUSH             0xFFF0F500
#define _LAVENDER_BLUSH            0xFFF0F500
#define _MISTYROSE                 0xFFE4E100
#define _MISTY_ROSE                0xFFE4E100
#define _WHITE                     0xFFFFFF00
#define _BLACK                     0x00000000
#define _DARKSLATEGRAY             0x2F4F4F00
#define _DARK_SLATE_GRAY           0x2F4F4F00
#define _DARKSLATEGREY             0x2F4F4F00
#define _DARK_SLATE_GREY           0x2F4F4F00
#define _DIMGRAY                   0x69696900
#define _DIM_GRAY                  0x69696900
#define _DIMGREY                   0x69696900
#define _DIM_GREY                  0x69696900
#define _SLATEGRAY                 0x70809000
#define _SLATE_GRAY                0x70809000
#define _SLATEGREY                 0x70809000
#define _SLATE_GREY                0x70809000
#define _LIGHTSLATEGRAY            0x77889900
#define _LIGHT_SLATE_GRAY          0x77889900
#define _LIGHTSLATEGREY            0x77889900
#define _LIGHT_SLATE_GREY          0x77889900
#define _GRAY                      0xBEBEBE00
#define _GREY                      0xBEBEBE00
#define _LIGHTGREY                 0xD3D3D300
#define _LIGHT_GREY                0xD3D3D300
#define _LIGHTGRAY                 0xD3D3D300
#define _LIGHT_GRAY                0xD3D3D300
#define _MIDNIGHTBLUE              0x19197000
#define _MIDNIGHT_BLUE             0x19197000
#define _NAVY                      0x00008000
#define _NAVYBLUE                  0x00008000
#define _NAVY_BLUE                 0x00008000
#define _CORNFLOWERBLUE            0x6495ED00
#define _CORNFLOWER_BLUE           0x6495ED00
#define _DARKSLATEBLUE             0x483D8B00
#define _DARK_SLATE_BLUE           0x483D8B00
#define _SLATEBLUE                 0x6A5ACD00
#define _SLATE_BLUE                0x6A5ACD00
#define _MEDIUMSLATEBLUE           0x7B68EE00
#define _MEDIUM_SLATE_BLUE         0x7B68EE00
#define _LIGHTSLATEBLUE            0x8470FF00
#define _LIGHT_SLATE_BLUE          0x8470FF00
#define _MEDIUMBLUE                0x0000CD00
#define _MEDIUM_BLUE               0x0000CD00
#define _ROYALBLUE                 0x4169E100
#define _ROYAL_BLUE                0x4169E100
#define _BLUE                      0x0000FF00
#define _DODGERBLUE                0x1E90FF00
#define _DODGER_BLUE               0x1E90FF00
#define _DEEPSKYBLUE               0x00BFFF00
#define _DEEP_SKY_BLUE             0x00BFFF00
#define _SKYBLUE                   0x87CEEB00
#define _SKY_BLUE                  0x87CEEB00
#define _LIGHTSKYBLUE              0x87CEFA00
#define _LIGHT_SKY_BLUE            0x87CEFA00
#define _STEELBLUE                 0x4682B400
#define _STEEL_BLUE                0x4682B400
#define _LIGHTSTEELBLUE            0xB0C4DE00
#define _LIGHT_STEEL_BLUE          0xB0C4DE00
#define _LIGHTBLUE                 0xADD8E600
#define _LIGHT_BLUE                0xADD8E600
#define _POWDERBLUE                0xB0E0E600
#define _POWDER_BLUE               0xB0E0E600
#define _PALETURQUOISE             0xAFEEEE00
#define _PALE_TURQUOISE            0xAFEEEE00
#define _DARKTURQUOISE             0x00CED100
#define _DARK_TURQUOISE            0x00CED100
#define _MEDIUMTURQUOISE           0x48D1CC00
#define _MEDIUM_TURQUOISE          0x48D1CC00
#define _TURQUOISE                 0x40E0D000
#define _CYAN                      0x00FFFF00
#define _LIGHTCYAN                 0xE0FFFF00
#define _LIGHT_CYAN                0xE0FFFF00
#define _CADETBLUE                 0x5F9EA000
#define _CADET_BLUE                0x5F9EA000
#define _MEDIUMAQUAMARINE          0x66CDAA00
#define _MEDIUM_AQUAMARINE         0x66CDAA00
#define _AQUAMARINE                0x7FFFD400
#define _DARKGREEN                 0x00640000
#define _DARK_GREEN                0x00640000
#define _DARKOLIVEGREEN            0x556B2F00
#define _DARK_OLIVE_GREEN          0x556B2F00
#define _DARKSEAGREEN              0x8FBC8F00
#define _DARK_SEA_GREEN            0x8FBC8F00
#define _SEAGREEN                  0x2E8B5700
#define _SEA_GREEN                 0x2E8B5700
#define _MEDIUMSEAGREEN            0x3CB37100
#define _MEDIUM_SEA_GREEN          0x3CB37100
#define _LIGHTSEAGREEN             0x20B2AA00
#define _LIGHT_SEA_GREEN           0x20B2AA00
#define _PALEGREEN                 0x98FB9800
#define _PALE_GREEN                0x98FB9800
#define _SPRINGGREEN               0x00FF7F00
#define _SPRING_GREEN              0x00FF7F00
#define _LAWNGREEN                 0x7CFC0000
#define _LAWN_GREEN                0x7CFC0000
#define _GREEN                     0x00FF0000
#define _CHARTREUSE                0x7FFF0000
#define _MEDIUMSPRINGGREEN         0x00FA9A00
#define _MEDIUM_SPRING_GREEN       0x00FA9A00
#define _GREENYELLOW               0xADFF2F00
#define _GREEN_YELLOW              0xADFF2F00
#define _LIMEGREEN                 0x32CD3200
#define _LIME_GREEN                0x32CD3200
#define _YELLOWGREEN               0x9ACD3200
#define _YELLOW_GREEN              0x9ACD3200
#define _FORESTGREEN               0x228B2200
#define _FOREST_GREEN              0x228B2200
#define _OLIVEDRAB                 0x6B8E2300
#define _OLIVE_DRAB                0x6B8E2300
#define _DARKKHAKI                 0xBDB76B00
#define _DARK_KHAKI                0xBDB76B00
#define _KHAKI                     0xF0E68C00
#define _PALEGOLDENROD             0xEEE8AA00
#define _PALE_GOLDENROD            0xEEE8AA00
#define _LIGHTGOLDENRODYELLOW      0xFAFAD200
#define _LIGHT_GOLDENROD_YELLOW    0xFAFAD200
#define _LIGHTYELLOW               0xFFFFE000
#define _LIGHT_YELLOW              0xFFFFE000
#define _YELLOW                    0xFFFF0000
#define _GOLD                      0xFFD70000
#define _LIGHTGOLDENROD            0xEEDD8200
#define _LIGHT_GOLDENROD           0xEEDD8200
#define _GOLDENROD                 0xDAA52000
#define _DARKGOLDENROD             0xB8860B00
#define _DARK_GOLDENROD            0xB8860B00
#define _ROSYBROWN                 0xBC8F8F00
#define _ROSY_BROWN                0xBC8F8F00
#define _INDIANRED                 0xCD5C5C00
#define _INDIAN_RED                0xCD5C5C00
#define _SADDLEBROWN               0x8B451300
#define _SADDLE_BROWN              0x8B451300
#define _SIENNA                    0xA0522D00
#define _PERU                      0xCD853F00
#define _BURLYWOOD                 0xDEB88700
#define _BEIGE                     0xF5F5DC00
#define _WHEAT                     0xF5DEB300
#define _SANDYBROWN                0xF4A46000
#define _SANDY_BROWN               0xF4A46000
#define _TAN                       0xD2B48C00
#define _CHOCOLATE                 0xD2691E00
#define _FIREBRICK                 0xB2222200
#define _BROWN                     0xA52A2A00
#define _DARKSALMON                0xE9967A00
#define _DARK_SALMON               0xE9967A00
#define _SALMON                    0xFA807200
#define _LIGHTSALMON               0xFFA07A00
#define _LIGHT_SALMON              0xFFA07A00
#define _ORANGE                    0xFFA50000
#define _DARKORANGE                0xFF8C0000
#define _DARK_ORANGE               0xFF8C0000
#define _CORAL                     0xFF7F5000
#define _LIGHTCORAL                0xF0808000
#define _LIGHT_CORAL               0xF0808000
#define _TOMATO                    0xFF634700
#define _ORANGERED                 0xFF450000
#define _ORANGE_RED                0xFF450000
#define _RED                       0xFF000000
#define _HOTPINK                   0xFF69B400
#define _HOT_PINK                  0xFF69B400
#define _DEEPPINK                  0xFF149300
#define _DEEP_PINK                 0xFF149300
#define _PINK                      0xFFC0CB00
#define _LIGHTPINK                 0xFFB6C100
#define _LIGHT_PINK                0xFFB6C100
#define _PALEVIOLETRED             0xDB709300
#define _PALE_VIOLET_RED           0xDB709300
#define _MAROON                    0xB0306000
#define _MEDIUMVIOLETRED           0xC7158500
#define _MEDIUM_VIOLET_RED         0xC7158500
#define _VIOLETRED                 0xD0209000
#define _VIOLET_RED                0xD0209000
#define _MAGENTA                   0xFF00FF00
#define _VIOLET                    0xEE82EE00
#define _PLUM                      0xDDA0DD00
#define _ORCHID                    0xDA70D600
#define _MEDIUMORCHID              0xBA55D300
#define _MEDIUM_ORCHID             0xBA55D300
#define _DARKORCHID                0x9932CC00
#define _DARK_ORCHID               0x9932CC00
#define _DARKVIOLET                0x9400D300
#define _DARK_VIOLET               0x9400D300
#define _BLUEVIOLET                0x8A2BE200
#define _BLUE_VIOLET               0x8A2BE200
#define _PURPLE                    0xA020F000
#define _MEDIUMPURPLE              0x9370DB00
#define _MEDIUM_PURPLE             0x9370DB00
#define _THISTLE                   0xD8BFD800
#define _SNOW1                     0xFFFAFA00
#define _SNOW_1                    0xFFFAFA00
#define _SNOW2                     0xEEE9E900
#define _SNOW_2                    0xEEE9E900
#define _SNOW3                     0xCDC9C900
#define _SNOW_3                    0xCDC9C900
#define _SNOW4                     0x8B898900
#define _SNOW_4                    0x8B898900
#define _SEASHELL1                 0xFFF5EE00
#define _SEASHELL_1                0xFFF5EE00
#define _SEASHELL2                 0xEEE5DE00
#define _SEASHELL_2                0xEEE5DE00
#define _SEASHELL3                 0xCDC5BF00
#define _SEASHELL_3                0xCDC5BF00
#define _SEASHELL4                 0x8B868200
#define _SEASHELL_4                0x8B868200
#define _ANTIQUEWHITE1             0xFFEFDB00
#define _ANTIQUE_WHITE_1           0xFFEFDB00
#define _ANTIQUEWHITE2             0xEEDFCC00
#define _ANTIQUE_WHITE_2           0xEEDFCC00
#define _ANTIQUEWHITE3             0xCDC0B000
#define _ANTIQUE_WHITE_3           0xCDC0B000
#define _ANTIQUEWHITE4             0x8B837800
#define _ANTIQUE_WHITE_4           0x8B837800
#define _BISQUE1                   0xFFE4C400
#define _BISQUE_1                  0xFFE4C400
#define _BISQUE2                   0xEED5B700
#define _BISQUE_2                  0xEED5B700
#define _BISQUE3                   0xCDB79E00
#define _BISQUE_3                  0xCDB79E00
#define _BISQUE4                   0x8B7D6B00
#define _BISQUE_4                  0x8B7D6B00
#define _PEACHPUFF1                0xFFDAB900
#define _PEACH_PUFF_1              0xFFDAB900
#define _PEACHPUFF2                0xEECBAD00
#define _PEACH_PUFF_2              0xEECBAD00
#define _PEACHPUFF3                0xCDAF9500
#define _PEACH_PUFF_3              0xCDAF9500
#define _PEACHPUFF4                0x8B776500
#define _PEACH_PUFF_4              0x8B776500
#define _NAVAJOWHITE1              0xFFDEAD00
#define _NAVAJO_WHITE_1            0xFFDEAD00
#define _NAVAJOWHITE2              0xEECFA100
#define _NAVAJO_WHITE_2            0xEECFA100
#define _NAVAJOWHITE3              0xCDB38B00
#define _NAVAJO_WHITE_3            0xCDB38B00
#define _NAVAJOWHITE4              0x8B795E00
#define _NAVAJO_WHITE_4            0x8B795E00
#define _LEMONCHIFFON1             0xFFFACD00
#define _LEMON_CHIFFON_1           0xFFFACD00
#define _LEMONCHIFFON2             0xEEE9BF00
#define _LEMON_CHIFFON_2           0xEEE9BF00
#define _LEMONCHIFFON3             0xCDC9A500
#define _LEMON_CHIFFON_3           0xCDC9A500
#define _LEMONCHIFFON4             0x8B897000
#define _LEMON_CHIFFON_4           0x8B897000
#define _CORNSILK1                 0xFFF8DC00
#define _CORNSILK_1                0xFFF8DC00
#define _CORNSILK2                 0xEEE8CD00
#define _CORNSILK_2                0xEEE8CD00
#define _CORNSILK3                 0xCDC8B100
#define _CORNSILK_3                0xCDC8B100
#define _CORNSILK4                 0x8B887800
#define _CORNSILK_4                0x8B887800
#define _IVORY1                    0xFFFFF000
#define _IVORY_1                   0xFFFFF000
#define _IVORY2                    0xEEEEE000
#define _IVORY_2                   0xEEEEE000
#define _IVORY3                    0xCDCDC100
#define _IVORY_3                   0xCDCDC100
#define _IVORY4                    0x8B8B8300
#define _IVORY_4                   0x8B8B8300
#define _HONEYDEW1                 0xF0FFF000
#define _HONEYDEW_1                0xF0FFF000
#define _HONEYDEW2                 0xE0EEE000
#define _HONEYDEW_2                0xE0EEE000
#define _HONEYDEW3                 0xC1CDC100
#define _HONEYDEW_3                0xC1CDC100
#define _HONEYDEW4                 0x838B8300
#define _HONEYDEW_4                0x838B8300
#define _LAVENDERBLUSH1            0xFFF0F500
#define _LAVENDER_BLUSH_1          0xFFF0F500
#define _LAVENDERBLUSH2            0xEEE0E500
#define _LAVENDER_BLUSH_2          0xEEE0E500
#define _LAVENDERBLUSH3            0xCDC1C500
#define _LAVENDER_BLUSH_3          0xCDC1C500
#define _LAVENDERBLUSH4            0x8B838600
#define _LAVENDER_BLUSH_4          0x8B838600
#define _MISTYROSE1                0xFFE4E100
#define _MISTY_ROSE_1              0xFFE4E100
#define _MISTYROSE2                0xEED5D200
#define _MISTY_ROSE_2              0xEED5D200
#define _MISTYROSE3                0xCDB7B500
#define _MISTY_ROSE_3              0xCDB7B500
#define _MISTYROSE4                0x8B7D7B00
#define _MISTY_ROSE_4              0x8B7D7B00
#define _AZURE1                    0xF0FFFF00
#define _AZURE_1                   0xF0FFFF00
#define _AZURE2                    0xE0EEEE00
#define _AZURE_2                   0xE0EEEE00
#define _AZURE3                    0xC1CDCD00
#define _AZURE_3                   0xC1CDCD00
#define _AZURE4                    0x838B8B00
#define _AZURE_4                   0x838B8B00
#define _SLATEBLUE1                0x836FFF00
#define _SLATE_BLUE_1              0x836FFF00
#define _SLATEBLUE2                0x7A67EE00
#define _SLATE_BLUE_2              0x7A67EE00
#define _SLATEBLUE3                0x6959CD00
#define _SLATE_BLUE_3              0x6959CD00
#define _SLATEBLUE4                0x473C8B00
#define _SLATE_BLUE_4              0x473C8B00
#define _ROYALBLUE1                0x4876FF00
#define _ROYAL_BLUE_1              0x4876FF00
#define _ROYALBLUE2                0x436EEE00
#define _ROYAL_BLUE_2              0x436EEE00
#define _ROYALBLUE3                0x3A5FCD00
#define _ROYAL_BLUE_3              0x3A5FCD00
#define _ROYALBLUE4                0x27408B00
#define _ROYAL_BLUE_4              0x27408B00
#define _BLUE1                     0x0000FF00
#define _BLUE_1                    0x0000FF00
#define _BLUE2                     0x0000EE00
#define _BLUE_2                    0x0000EE00
#define _BLUE3                     0x0000CD00
#define _BLUE_3                    0x0000CD00
#define _BLUE4                     0x00008B00
#define _BLUE_4                    0x00008B00
#define _DODGERBLUE1               0x1E90FF00
#define _DODGER_BLUE_1             0x1E90FF00
#define _DODGERBLUE2               0x1C86EE00
#define _DODGER_BLUE_2             0x1C86EE00
#define _DODGERBLUE3               0x1874CD00
#define _DODGER_BLUE_3             0x1874CD00
#define _DODGERBLUE4               0x104E8B00
#define _DODGER_BLUE_4             0x104E8B00
#define _STEELBLUE1                0x63B8FF00
#define _STEEL_BLUE_1              0x63B8FF00
#define _STEELBLUE2                0x5CACEE00
#define _STEEL_BLUE_2              0x5CACEE00
#define _STEELBLUE3                0x4F94CD00
#define _STEEL_BLUE_3              0x4F94CD00
#define _STEELBLUE4                0x36648B00
#define _STEEL_BLUE_4              0x36648B00
#define _DEEPSKYBLUE1              0x00BFFF00
#define _DEEP_SKY_BLUE_1           0x00BFFF00
#define _DEEPSKYBLUE2              0x00B2EE00
#define _DEEP_SKY_BLUE_2           0x00B2EE00
#define _DEEPSKYBLUE3              0x009ACD00
#define _DEEP_SKY_BLUE_3           0x009ACD00
#define _DEEPSKYBLUE4              0x00688B00
#define _DEEP_SKY_BLUE_4           0x00688B00
#define _SKYBLUE1                  0x87CEFF00
#define _SKY_BLUE_1                0x87CEFF00
#define _SKYBLUE2                  0x7EC0EE00
#define _SKY_BLUE_2                0x7EC0EE00
#define _SKYBLUE3                  0x6CA6CD00
#define _SKY_BLUE_3                0x6CA6CD00
#define _SKYBLUE4                  0x4A708B00
#define _SKY_BLUE_4                0x4A708B00
#define _LIGHTSKYBLUE1             0xB0E2FF00
#define _LIGHT_SKY_BLUE_1          0xB0E2FF00
#define _LIGHTSKYBLUE2             0xA4D3EE00
#define _LIGHT_SKY_BLUE_2          0xA4D3EE00
#define _LIGHTSKYBLUE3             0x8DB6CD00
#define _LIGHT_SKY_BLUE_3          0x8DB6CD00
#define _LIGHTSKYBLUE4             0x607B8B00
#define _LIGHT_SKY_BLUE_4          0x607B8B00
#define _SLATEGRAY1                0xC6E2FF00
#define _SLATE_GRAY_1              0xC6E2FF00
#define _SLATEGRAY2                0xB9D3EE00
#define _SLATE_GRAY_2              0xB9D3EE00
#define _SLATEGRAY3                0x9FB6CD00
#define _SLATE_GRAY_3              0x9FB6CD00
#define _SLATEGRAY4                0x6C7B8B00
#define _SLATE_GRAY_4              0x6C7B8B00
#define _SLATEGREY1                0xC6E2FF00
#define _SLATE_GREY_1              0xC6E2FF00
#define _SLATEGREY2                0xB9D3EE00
#define _SLATE_GREY_2              0xB9D3EE00
#define _SLATEGREY3                0x9FB6CD00
#define _SLATE_GREY_3              0x9FB6CD00
#define _SLATEGREY4                0x6C7B8B00
#define _SLATE_GREY_4              0x6C7B8B00
#define _LIGHTSTEELBLUE1           0xCAE1FF00
#define _LIGHT_STEEL_BLUE_1        0xCAE1FF00
#define _LIGHTSTEELBLUE2           0xBCD2EE00
#define _LIGHT_STEEL_BLUE_2        0xBCD2EE00
#define _LIGHTSTEELBLUE3           0xA2B5CD00
#define _LIGHT_STEEL_BLUE_3        0xA2B5CD00
#define _LIGHTSTEELBLUE4           0x6E7B8B00
#define _LIGHT_STEEL_BLUE_4        0x6E7B8B00
#define _LIGHTBLUE1                0xBFEFFF00
#define _LIGHT_BLUE_1              0xBFEFFF00
#define _LIGHTBLUE2                0xB2DFEE00
#define _LIGHT_BLUE_2              0xB2DFEE00
#define _LIGHTBLUE3                0x9AC0CD00
#define _LIGHT_BLUE_3              0x9AC0CD00
#define _LIGHTBLUE4                0x68838B00
#define _LIGHT_BLUE_4              0x68838B00
#define _LIGHTCYAN1                0xE0FFFF00
#define _LIGHT_CYAN_1              0xE0FFFF00
#define _LIGHTCYAN2                0xD1EEEE00
#define _LIGHT_CYAN_2              0xD1EEEE00
#define _LIGHTCYAN3                0xB4CDCD00
#define _LIGHT_CYAN_3              0xB4CDCD00
#define _LIGHTCYAN4                0x7A8B8B00
#define _LIGHT_CYAN_4              0x7A8B8B00
#define _PALETURQUOISE1            0xBBFFFF00
#define _PALE_TURQUOISE_1          0xBBFFFF00
#define _PALETURQUOISE2            0xAEEEEE00
#define _PALE_TURQUOISE_2          0xAEEEEE00
#define _PALETURQUOISE3            0x96CDCD00
#define _PALE_TURQUOISE_3          0x96CDCD00
#define _PALETURQUOISE4            0x668B8B00
#define _PALE_TURQUOISE_4          0x668B8B00
#define _CADETBLUE1                0x98F5FF00
#define _CADET_BLUE_1              0x98F5FF00
#define _CADETBLUE2                0x8EE5EE00
#define _CADET_BLUE_2              0x8EE5EE00
#define _CADETBLUE3                0x7AC5CD00
#define _CADET_BLUE_3              0x7AC5CD00
#define _CADETBLUE4                0x53868B00
#define _CADET_BLUE_4              0x53868B00
#define _TURQUOISE1                0x00F5FF00
#define _TURQUOISE_1               0x00F5FF00
#define _TURQUOISE2                0x00E5EE00
#define _TURQUOISE_2               0x00E5EE00
#define _TURQUOISE3                0x00C5CD00
#define _TURQUOISE_3               0x00C5CD00
#define _TURQUOISE4                0x00868B00
#define _TURQUOISE_4               0x00868B00
#define _CYAN1                     0x00FFFF00
#define _CYAN_1                    0x00FFFF00
#define _CYAN2                     0x00EEEE00
#define _CYAN_2                    0x00EEEE00
#define _CYAN3                     0x00CDCD00
#define _CYAN_3                    0x00CDCD00
#define _CYAN4                     0x008B8B00
#define _CYAN_4                    0x008B8B00
#define _DARKSLATEGRAY1            0x97FFFF00
#define _DARK_SLATE_GRAY_1         0x97FFFF00
#define _DARKSLATEGRAY2            0x8DEEEE00
#define _DARK_SLATE_GRAY_2         0x8DEEEE00
#define _DARKSLATEGRAY3            0x79CDCD00
#define _DARK_SLATE_GRAY_3         0x79CDCD00
#define _DARKSLATEGRAY4            0x528B8B00
#define _DARK_SLATE_GRAY_4         0x528B8B00
#define _DARKSLATEGREY1            0x97FFFF00
#define _DARK_SLATE_GREY_1         0x97FFFF00
#define _DARKSLATEGREY2            0x8DEEEE00
#define _DARK_SLATE_GREY_2         0x8DEEEE00
#define _DARKSLATEGREY3            0x79CDCD00
#define _DARK_SLATE_GREY_3         0x79CDCD00
#define _DARKSLATEGREY4            0x528B8B00
#define _DARK_SLATE_GREY_4         0x528B8B00
#define _AQUAMARINE1               0x7FFFD400
#define _AQUAMARINE_1              0x7FFFD400
#define _AQUAMARINE2               0x76EEC600
#define _AQUAMARINE_2              0x76EEC600
#define _AQUAMARINE3               0x66CDAA00
#define _AQUAMARINE_3              0x66CDAA00
#define _AQUAMARINE4               0x458B7400
#define _AQUAMARINE_4              0x458B7400
#define _DARKSEAGREEN1             0xC1FFC100
#define _DARK_SEA_GREEN_1          0xC1FFC100
#define _DARKSEAGREEN2             0xB4EEB400
#define _DARK_SEA_GREEN_2          0xB4EEB400
#define _DARKSEAGREEN3             0x9BCD9B00
#define _DARK_SEA_GREEN_3          0x9BCD9B00
#define _DARKSEAGREEN4             0x698B6900
#define _DARK_SEA_GREEN_4          0x698B6900
#define _SEAGREEN1                 0x54FF9F00
#define _SEA_GREEN_1               0x54FF9F00
#define _SEAGREEN2                 0x4EEE9400
#define _SEA_GREEN_2               0x4EEE9400
#define _SEAGREEN3                 0x43CD8000
#define _SEA_GREEN_3               0x43CD8000
#define _SEAGREEN4                 0x2E8B5700
#define _SEA_GREEN_4               0x2E8B5700
#define _PALEGREEN1                0x9AFF9A00
#define _PALE_GREEN_1              0x9AFF9A00
#define _PALEGREEN2                0x90EE9000
#define _PALE_GREEN_2              0x90EE9000
#define _PALEGREEN3                0x7CCD7C00
#define _PALE_GREEN_3              0x7CCD7C00
#define _PALEGREEN4                0x548B5400
#define _PALE_GREEN_4              0x548B5400
#define _SPRINGGREEN1              0x00FF7F00
#define _SPRING_GREEN_1            0x00FF7F00
#define _SPRINGGREEN2              0x00EE7600
#define _SPRING_GREEN_2            0x00EE7600
#define _SPRINGGREEN3              0x00CD6600
#define _SPRING_GREEN_3            0x00CD6600
#define _SPRINGGREEN4              0x008B4500
#define _SPRING_GREEN_4            0x008B4500
#define _GREEN1                    0x00FF0000
#define _GREEN_1                   0x00FF0000
#define _GREEN2                    0x00EE0000
#define _GREEN_2                   0x00EE0000
#define _GREEN3                    0x00CD0000
#define _GREEN_3                   0x00CD0000
#define _GREEN4                    0x008B0000
#define _GREEN_4                   0x008B0000
#define _CHARTREUSE1               0x7FFF0000
#define _CHARTREUSE_1              0x7FFF0000
#define _CHARTREUSE2               0x76EE0000
#define _CHARTREUSE_2              0x76EE0000
#define _CHARTREUSE3               0x66CD0000
#define _CHARTREUSE_3              0x66CD0000
#define _CHARTREUSE4               0x458B0000
#define _CHARTREUSE_4              0x458B0000
#define _OLIVEDRAB1                0xC0FF3E00
#define _OLIVE_DRAB_1              0xC0FF3E00
#define _OLIVEDRAB2                0xB3EE3A00
#define _OLIVE_DRAB_2              0xB3EE3A00
#define _OLIVEDRAB3                0x9ACD3200
#define _OLIVE_DRAB_3              0x9ACD3200
#define _OLIVEDRAB4                0x698B2200
#define _OLIVE_DRAB_4              0x698B2200
#define _DARKOLIVEGREEN1           0xCAFF7000
#define _DARK_OLIVE_GREEN_1        0xCAFF7000
#define _DARKOLIVEGREEN2           0xBCEE6800
#define _DARK_OLIVE_GREEN_2        0xBCEE6800
#define _DARKOLIVEGREEN3           0xA2CD5A00
#define _DARK_OLIVE_GREEN_3        0xA2CD5A00
#define _DARKOLIVEGREEN4           0x6E8B3D00
#define _DARK_OLIVE_GREEN_4        0x6E8B3D00
#define _KHAKI1                    0xFFF68F00
#define _KHAKI_1                   0xFFF68F00
#define _KHAKI2                    0xEEE68500
#define _KHAKI_2                   0xEEE68500
#define _KHAKI3                    0xCDC67300
#define _KHAKI_3                   0xCDC67300
#define _KHAKI4                    0x8B864E00
#define _KHAKI_4                   0x8B864E00
#define _LIGHTGOLDENROD1           0xFFEC8B00
#define _LIGHT_GOLDENROD_1         0xFFEC8B00
#define _LIGHTGOLDENROD2           0xEEDC8200
#define _LIGHT_GOLDENROD_2         0xEEDC8200
#define _LIGHTGOLDENROD3           0xCDBE7000
#define _LIGHT_GOLDENROD_3         0xCDBE7000
#define _LIGHTGOLDENROD4           0x8B814C00
#define _LIGHT_GOLDENROD_4         0x8B814C00
#define _LIGHTYELLOW1              0xFFFFE000
#define _LIGHT_YELLOW_1            0xFFFFE000
#define _LIGHTYELLOW2              0xEEEED100
#define _LIGHT_YELLOW_2            0xEEEED100
#define _LIGHTYELLOW3              0xCDCDB400
#define _LIGHT_YELLOW_3            0xCDCDB400
#define _LIGHTYELLOW4              0x8B8B7A00
#define _LIGHT_YELLOW_4            0x8B8B7A00
#define _YELLOW1                   0xFFFF0000
#define _YELLOW_1                  0xFFFF0000
#define _YELLOW2                   0xEEEE0000
#define _YELLOW_2                  0xEEEE0000
#define _YELLOW3                   0xCDCD0000
#define _YELLOW_3                  0xCDCD0000
#define _YELLOW4                   0x8B8B0000
#define _YELLOW_4                  0x8B8B0000
#define _GOLD1                     0xFFD70000
#define _GOLD_1                    0xFFD70000
#define _GOLD2                     0xEEC90000
#define _GOLD_2                    0xEEC90000
#define _GOLD3                     0xCDAD0000
#define _GOLD_3                    0xCDAD0000
#define _GOLD4                     0x8B750000
#define _GOLD_4                    0x8B750000
#define _GOLDENROD1                0xFFC12500
#define _GOLDENROD_1               0xFFC12500
#define _GOLDENROD2                0xEEB42200
#define _GOLDENROD_2               0xEEB42200
#define _GOLDENROD3                0xCD9B1D00
#define _GOLDENROD_3               0xCD9B1D00
#define _GOLDENROD4                0x8B691400
#define _GOLDENROD_4               0x8B691400
#define _DARKGOLDENROD1            0xFFB90F00
#define _DARK_GOLDENROD_1          0xFFB90F00
#define _DARKGOLDENROD2            0xEEAD0E00
#define _DARK_GOLDENROD_2          0xEEAD0E00
#define _DARKGOLDENROD3            0xCD950C00
#define _DARK_GOLDENROD_3          0xCD950C00
#define _DARKGOLDENROD4            0x8B650800
#define _DARK_GOLDENROD_4          0x8B650800
#define _ROSYBROWN1                0xFFC1C100
#define _ROSY_BROWN_1              0xFFC1C100
#define _ROSYBROWN2                0xEEB4B400
#define _ROSY_BROWN_2              0xEEB4B400
#define _ROSYBROWN3                0xCD9B9B00
#define _ROSY_BROWN_3              0xCD9B9B00
#define _ROSYBROWN4                0x8B696900
#define _ROSY_BROWN_4              0x8B696900
#define _INDIANRED1                0xFF6A6A00
#define _INDIAN_RED_1              0xFF6A6A00
#define _INDIANRED2                0xEE636300
#define _INDIAN_RED_2              0xEE636300
#define _INDIANRED3                0xCD555500
#define _INDIAN_RED_3              0xCD555500
#define _INDIANRED4                0x8B3A3A00
#define _INDIAN_RED_4              0x8B3A3A00
#define _SIENNA1                   0xFF824700
#define _SIENNA_1                  0xFF824700
#define _SIENNA2                   0xEE794200
#define _SIENNA_2                  0xEE794200
#define _SIENNA3                   0xCD683900
#define _SIENNA_3                  0xCD683900
#define _SIENNA4                   0x8B472600
#define _SIENNA_4                  0x8B472600
#define _BURLYWOOD1                0xFFD39B00
#define _BURLYWOOD_1               0xFFD39B00
#define _BURLYWOOD2                0xEEC59100
#define _BURLYWOOD_2               0xEEC59100
#define _BURLYWOOD3                0xCDAA7D00
#define _BURLYWOOD_3               0xCDAA7D00
#define _BURLYWOOD4                0x8B735500
#define _BURLYWOOD_4               0x8B735500
#define _WHEAT1                    0xFFE7BA00
#define _WHEAT_1                   0xFFE7BA00
#define _WHEAT2                    0xEED8AE00
#define _WHEAT_2                   0xEED8AE00
#define _WHEAT3                    0xCDBA9600
#define _WHEAT_3                   0xCDBA9600
#define _WHEAT4                    0x8B7E6600
#define _WHEAT_4                   0x8B7E6600
#define _TAN1                      0xFFA54F00
#define _TAN_1                     0xFFA54F00
#define _TAN2                      0xEE9A4900
#define _TAN_2                     0xEE9A4900
#define _TAN3                      0xCD853F00
#define _TAN_3                     0xCD853F00
#define _TAN4                      0x8B5A2B00
#define _TAN_4                     0x8B5A2B00
#define _CHOCOLATE1                0xFF7F2400
#define _CHOCOLATE_1               0xFF7F2400
#define _CHOCOLATE2                0xEE762100
#define _CHOCOLATE_2               0xEE762100
#define _CHOCOLATE3                0xCD661D00
#define _CHOCOLATE_3               0xCD661D00
#define _CHOCOLATE4                0x8B451300
#define _CHOCOLATE_4               0x8B451300
#define _FIREBRICK1                0xFF303000
#define _FIREBRICK_1               0xFF303000
#define _FIREBRICK2                0xEE2C2C00
#define _FIREBRICK_2               0xEE2C2C00
#define _FIREBRICK3                0xCD262600
#define _FIREBRICK_3               0xCD262600
#define _FIREBRICK4                0x8B1A1A00
#define _FIREBRICK_4               0x8B1A1A00
#define _BROWN1                    0xFF404000
#define _BROWN_1                   0xFF404000
#define _BROWN2                    0xEE3B3B00
#define _BROWN_2                   0xEE3B3B00
#define _BROWN3                    0xCD333300
#define _BROWN_3                   0xCD333300
#define _BROWN4                    0x8B232300
#define _BROWN_4                   0x8B232300
#define _SALMON1                   0xFF8C6900
#define _SALMON_1                  0xFF8C6900
#define _SALMON2                   0xEE826200
#define _SALMON_2                  0xEE826200
#define _SALMON3                   0xCD705400
#define _SALMON_3                  0xCD705400
#define _SALMON4                   0x8B4C3900
#define _SALMON_4                  0x8B4C3900
#define _LIGHTSALMON1              0xFFA07A00
#define _LIGHT_SALMON_1            0xFFA07A00
#define _LIGHTSALMON2              0xEE957200
#define _LIGHT_SALMON_2            0xEE957200
#define _LIGHTSALMON3              0xCD816200
#define _LIGHT_SALMON_3            0xCD816200
#define _LIGHTSALMON4              0x8B574200
#define _LIGHT_SALMON_4            0x8B574200
#define _ORANGE1                   0xFFA50000
#define _ORANGE_1                  0xFFA50000
#define _ORANGE2                   0xEE9A0000
#define _ORANGE_2                  0xEE9A0000
#define _ORANGE3                   0xCD850000
#define _ORANGE_3                  0xCD850000
#define _ORANGE4                   0x8B5A0000
#define _ORANGE_4                  0x8B5A0000
#define _DARKORANGE1               0xFF7F0000
#define _DARK_ORANGE_1             0xFF7F0000
#define _DARKORANGE2               0xEE760000
#define _DARK_ORANGE_2             0xEE760000
#define _DARKORANGE3               0xCD660000
#define _DARK_ORANGE_3             0xCD660000
#define _DARKORANGE4               0x8B450000
#define _DARK_ORANGE_4             0x8B450000
#define _CORAL1                    0xFF725600
#define _CORAL_1                   0xFF725600
#define _CORAL2                    0xEE6A5000
#define _CORAL_2                   0xEE6A5000
#define _CORAL3                    0xCD5B4500
#define _CORAL_3                   0xCD5B4500
#define _CORAL4                    0x8B3E2F00
#define _CORAL_4                   0x8B3E2F00
#define _TOMATO1                   0xFF634700
#define _TOMATO_1                  0xFF634700
#define _TOMATO2                   0xEE5C4200
#define _TOMATO_2                  0xEE5C4200
#define _TOMATO3                   0xCD4F3900
#define _TOMATO_3                  0xCD4F3900
#define _TOMATO4                   0x8B362600
#define _TOMATO_4                  0x8B362600
#define _ORANGERED1                0xFF450000
#define _ORANGE_RED_1              0xFF450000
#define _ORANGERED2                0xEE400000
#define _ORANGE_RED_2              0xEE400000
#define _ORANGERED3                0xCD370000
#define _ORANGE_RED_3              0xCD370000
#define _ORANGERED4                0x8B250000
#define _ORANGE_RED_4              0x8B250000
#define _RED1                      0xFF000000
#define _RED_1                     0xFF000000
#define _RED2                      0xEE000000
#define _RED_2                     0xEE000000
#define _RED3                      0xCD000000
#define _RED_3                     0xCD000000
#define _RED4                      0x8B000000
#define _RED_4                     0x8B000000
#define _DEEPPINK1                 0xFF149300
#define _DEEP_PINK_1               0xFF149300
#define _DEEPPINK2                 0xEE128900
#define _DEEP_PINK_2               0xEE128900
#define _DEEPPINK3                 0xCD107600
#define _DEEP_PINK_3               0xCD107600
#define _DEEPPINK4                 0x8B0A5000
#define _DEEP_PINK_4               0x8B0A5000
#define _HOTPINK1                  0xFF6EB400
#define _HOT_PINK_1                0xFF6EB400
#define _HOTPINK2                  0xEE6AA700
#define _HOT_PINK_2                0xEE6AA700
#define _HOTPINK3                  0xCD609000
#define _HOT_PINK_3                0xCD609000
#define _HOTPINK4                  0x8B3A6200
#define _HOT_PINK_4                0x8B3A6200
#define _PINK1                     0xFFB5C500
#define _PINK_1                    0xFFB5C500
#define _PINK2                     0xEEA9B800
#define _PINK_2                    0xEEA9B800
#define _PINK3                     0xCD919E00
#define _PINK_3                    0xCD919E00
#define _PINK4                     0x8B636C00
#define _PINK_4                    0x8B636C00
#define _LIGHTPINK1                0xFFAEB900
#define _LIGHT_PINK_1              0xFFAEB900
#define _LIGHTPINK2                0xEEA2AD00
#define _LIGHT_PINK_2              0xEEA2AD00
#define _LIGHTPINK3                0xCD8C9500
#define _LIGHT_PINK_3              0xCD8C9500
#define _LIGHTPINK4                0x8B5F6500
#define _LIGHT_PINK_4              0x8B5F6500
#define _PALEVIOLETRED1            0xFF82AB00
#define _PALE_VIOLET_RED_1         0xFF82AB00
#define _PALEVIOLETRED2            0xEE799F00
#define _PALE_VIOLET_RED_2         0xEE799F00
#define _PALEVIOLETRED3            0xCD688900
#define _PALE_VIOLET_RED_3         0xCD688900
#define _PALEVIOLETRED4            0x8B475D00
#define _PALE_VIOLET_RED_4         0x8B475D00
#define _MAROON1                   0xFF34B300
#define _MAROON_1                  0xFF34B300
#define _MAROON2                   0xEE30A700
#define _MAROON_2                  0xEE30A700
#define _MAROON3                   0xCD299000
#define _MAROON_3                  0xCD299000
#define _MAROON4                   0x8B1C6200
#define _MAROON_4                  0x8B1C6200
#define _VIOLETRED1                0xFF3E9600
#define _VIOLET_RED_1              0xFF3E9600
#define _VIOLETRED2                0xEE3A8C00
#define _VIOLET_RED_2              0xEE3A8C00
#define _VIOLETRED3                0xCD327800
#define _VIOLET_RED_3              0xCD327800
#define _VIOLETRED4                0x8B225200
#define _VIOLET_RED_4              0x8B225200
#define _MAGENTA1                  0xFF00FF00
#define _MAGENTA_1                 0xFF00FF00
#define _MAGENTA2                  0xEE00EE00
#define _MAGENTA_2                 0xEE00EE00
#define _MAGENTA3                  0xCD00CD00
#define _MAGENTA_3                 0xCD00CD00
#define _MAGENTA4                  0x8B008B00
#define _MAGENTA_4                 0x8B008B00
#define _ORCHID1                   0xFF83FA00
#define _ORCHID_1                  0xFF83FA00
#define _ORCHID2                   0xEE7AE900
#define _ORCHID_2                  0xEE7AE900
#define _ORCHID3                   0xCD69C900
#define _ORCHID_3                  0xCD69C900
#define _ORCHID4                   0x8B478900
#define _ORCHID_4                  0x8B478900
#define _PLUM1                     0xFFBBFF00
#define _PLUM_1                    0xFFBBFF00
#define _PLUM2                     0xEEAEEE00
#define _PLUM_2                    0xEEAEEE00
#define _PLUM3                     0xCD96CD00
#define _PLUM_3                    0xCD96CD00
#define _PLUM4                     0x8B668B00
#define _PLUM_4                    0x8B668B00
#define _MEDIUMORCHID1             0xE066FF00
#define _MEDIUM_ORCHID_1           0xE066FF00
#define _MEDIUMORCHID2             0xD15FEE00
#define _MEDIUM_ORCHID_2           0xD15FEE00
#define _MEDIUMORCHID3             0xB452CD00
#define _MEDIUM_ORCHID_3           0xB452CD00
#define _MEDIUMORCHID4             0x7A378B00
#define _MEDIUM_ORCHID_4           0x7A378B00
#define _DARKORCHID1               0xBF3EFF00
#define _DARK_ORCHID_1             0xBF3EFF00
#define _DARKORCHID2               0xB23AEE00
#define _DARK_ORCHID_2             0xB23AEE00
#define _DARKORCHID3               0x9A32CD00
#define _DARK_ORCHID_3             0x9A32CD00
#define _DARKORCHID4               0x68228B00
#define _DARK_ORCHID_4             0x68228B00
#define _PURPLE1                   0x9B30FF00
#define _PURPLE_1                  0x9B30FF00
#define _PURPLE2                   0x912CEE00
#define _PURPLE_2                  0x912CEE00
#define _PURPLE3                   0x7D26CD00
#define _PURPLE_3                  0x7D26CD00
#define _PURPLE4                   0x551A8B00
#define _PURPLE_4                  0x551A8B00
#define _MEDIUMPURPLE1             0xAB82FF00
#define _MEDIUM_PURPLE_1           0xAB82FF00
#define _MEDIUMPURPLE2             0x9F79EE00
#define _MEDIUM_PURPLE_2           0x9F79EE00
#define _MEDIUMPURPLE3             0x8968CD00
#define _MEDIUM_PURPLE_3           0x8968CD00
#define _MEDIUMPURPLE4             0x5D478B00
#define _MEDIUM_PURPLE_4           0x5D478B00
#define _THISTLE1                  0xFFE1FF00
#define _THISTLE_1                 0xFFE1FF00
#define _THISTLE2                  0xEED2EE00
#define _THISTLE_2                 0xEED2EE00
#define _THISTLE3                  0xCDB5CD00
#define _THISTLE_3                 0xCDB5CD00
#define _THISTLE4                  0x8B7B8B00
#define _THISTLE_4                 0x8B7B8B00
#define _GRAY0                     0x00000000
#define _GRAY_0                    0x00000000
#define _GREY0                     0x00000000
#define _GREY_0                    0x00000000
#define _GRAY1                     0x03030300
#define _GRAY_1                    0x03030300
#define _GREY1                     0x03030300
#define _GREY_1                    0x03030300
#define _GRAY2                     0x05050500
#define _GRAY_2                    0x05050500
#define _GREY2                     0x05050500
#define _GREY_2                    0x05050500
#define _GRAY3                     0x08080800
#define _GRAY_3                    0x08080800
#define _GREY3                     0x08080800
#define _GREY_3                    0x08080800
#define _GRAY4                     0x0A0A0A00
#define _GRAY_4                    0x0A0A0A00
#define _GREY4                     0x0A0A0A00
#define _GREY_4                    0x0A0A0A00
#define _GRAY5                     0x0D0D0D00
#define _GRAY_5                    0x0D0D0D00
#define _GREY5                     0x0D0D0D00
#define _GREY_5                    0x0D0D0D00
#define _GRAY6                     0x0F0F0F00
#define _GRAY_6                    0x0F0F0F00
#define _GREY6                     0x0F0F0F00
#define _GREY_6                    0x0F0F0F00
#define _GRAY7                     0x12121200
#define _GRAY_7                    0x12121200
#define _GREY7                     0x12121200
#define _GREY_7                    0x12121200
#define _GRAY8                     0x14141400
#define _GRAY_8                    0x14141400
#define _GREY8                     0x14141400
#define _GREY_8                    0x14141400
#define _GRAY9                     0x17171700
#define _GRAY_9                    0x17171700
#define _GREY9                     0x17171700
#define _GREY_9                    0x17171700
#define _GRAY10                    0x1A1A1A00
#define _GRAY_10                   0x1A1A1A00
#define _GREY10                    0x1A1A1A00
#define _GREY_10                   0x1A1A1A00
#define _GRAY11                    0x1C1C1C00
#define _GRAY_11                   0x1C1C1C00
#define _GREY11                    0x1C1C1C00
#define _GREY_11                   0x1C1C1C00
#define _GRAY12                    0x1F1F1F00
#define _GRAY_12                   0x1F1F1F00
#define _GREY12                    0x1F1F1F00
#define _GREY_12                   0x1F1F1F00
#define _GRAY13                    0x21212100
#define _GRAY_13                   0x21212100
#define _GREY13                    0x21212100
#define _GREY_13                   0x21212100
#define _GRAY14                    0x24242400
#define _GRAY_14                   0x24242400
#define _GREY14                    0x24242400
#define _GREY_14                   0x24242400
#define _GRAY15                    0x26262600
#define _GRAY_15                   0x26262600
#define _GREY15                    0x26262600
#define _GREY_15                   0x26262600
#define _GRAY16                    0x29292900
#define _GRAY_16                   0x29292900
#define _GREY16                    0x29292900
#define _GREY_16                   0x29292900
#define _GRAY17                    0x2B2B2B00
#define _GRAY_17                   0x2B2B2B00
#define _GREY17                    0x2B2B2B00
#define _GREY_17                   0x2B2B2B00
#define _GRAY18                    0x2E2E2E00
#define _GRAY_18                   0x2E2E2E00
#define _GREY18                    0x2E2E2E00
#define _GREY_18                   0x2E2E2E00
#define _GRAY19                    0x30303000
#define _GRAY_19                   0x30303000
#define _GREY19                    0x30303000
#define _GREY_19                   0x30303000
#define _GRAY20                    0x33333300
#define _GRAY_20                   0x33333300
#define _GREY20                    0x33333300
#define _GREY_20                   0x33333300
#define _GRAY21                    0x36363600
#define _GRAY_21                   0x36363600
#define _GREY21                    0x36363600
#define _GREY_21                   0x36363600
#define _GRAY22                    0x38383800
#define _GRAY_22                   0x38383800
#define _GREY22                    0x38383800
#define _GREY_22                   0x38383800
#define _GRAY23                    0x3B3B3B00
#define _GRAY_23                   0x3B3B3B00
#define _GREY23                    0x3B3B3B00
#define _GREY_23                   0x3B3B3B00
#define _GRAY24                    0x3D3D3D00
#define _GRAY_24                   0x3D3D3D00
#define _GREY24                    0x3D3D3D00
#define _GREY_24                   0x3D3D3D00
#define _GRAY25                    0x40404000
#define _GRAY_25                   0x40404000
#define _GREY25                    0x40404000
#define _GREY_25                   0x40404000
#define _GRAY26                    0x42424200
#define _GRAY_26                   0x42424200
#define _GREY26                    0x42424200
#define _GREY_26                   0x42424200
#define _GRAY27                    0x45454500
#define _GRAY_27                   0x45454500
#define _GREY27                    0x45454500
#define _GREY_27                   0x45454500
#define _GRAY28                    0x47474700
#define _GRAY_28                   0x47474700
#define _GREY28                    0x47474700
#define _GREY_28                   0x47474700
#define _GRAY29                    0x4A4A4A00
#define _GRAY_29                   0x4A4A4A00
#define _GREY29                    0x4A4A4A00
#define _GREY_29                   0x4A4A4A00
#define _GRAY30                    0x4D4D4D00
#define _GRAY_30                   0x4D4D4D00
#define _GREY30                    0x4D4D4D00
#define _GREY_30                   0x4D4D4D00
#define _GRAY31                    0x4F4F4F00
#define _GRAY_31                   0x4F4F4F00
#define _GREY31                    0x4F4F4F00
#define _GREY_31                   0x4F4F4F00
#define _GRAY32                    0x52525200
#define _GRAY_32                   0x52525200
#define _GREY32                    0x52525200
#define _GREY_32                   0x52525200
#define _GRAY33                    0x54545400
#define _GRAY_33                   0x54545400
#define _GREY33                    0x54545400
#define _GREY_33                   0x54545400
#define _GRAY34                    0x57575700
#define _GRAY_34                   0x57575700
#define _GREY34                    0x57575700
#define _GREY_34                   0x57575700
#define _GRAY35                    0x59595900
#define _GRAY_35                   0x59595900
#define _GREY35                    0x59595900
#define _GREY_35                   0x59595900
#define _GRAY36                    0x5C5C5C00
#define _GRAY_36                   0x5C5C5C00
#define _GREY36                    0x5C5C5C00
#define _GREY_36                   0x5C5C5C00
#define _GRAY37                    0x5E5E5E00
#define _GRAY_37                   0x5E5E5E00
#define _GREY37                    0x5E5E5E00
#define _GREY_37                   0x5E5E5E00
#define _GRAY38                    0x61616100
#define _GRAY_38                   0x61616100
#define _GREY38                    0x61616100
#define _GREY_38                   0x61616100
#define _GRAY39                    0x63636300
#define _GRAY_39                   0x63636300
#define _GREY39                    0x63636300
#define _GREY_39                   0x63636300
#define _GRAY40                    0x66666600
#define _GRAY_40                   0x66666600
#define _GREY40                    0x66666600
#define _GREY_40                   0x66666600
#define _GRAY41                    0x69696900
#define _GRAY_41                   0x69696900
#define _GREY41                    0x69696900
#define _GREY_41                   0x69696900
#define _GRAY42                    0x6B6B6B00
#define _GRAY_42                   0x6B6B6B00
#define _GREY42                    0x6B6B6B00
#define _GREY_42                   0x6B6B6B00
#define _GRAY43                    0x6E6E6E00
#define _GRAY_43                   0x6E6E6E00
#define _GREY43                    0x6E6E6E00
#define _GREY_43                   0x6E6E6E00
#define _GRAY44                    0x70707000
#define _GRAY_44                   0x70707000
#define _GREY44                    0x70707000
#define _GREY_44                   0x70707000
#define _GRAY45                    0x73737300
#define _GRAY_45                   0x73737300
#define _GREY45                    0x73737300
#define _GREY_45                   0x73737300
#define _GRAY46                    0x75757500
#define _GRAY_46                   0x75757500
#define _GREY46                    0x75757500
#define _GREY_46                   0x75757500
#define _GRAY47                    0x78787800
#define _GRAY_47                   0x78787800
#define _GREY47                    0x78787800
#define _GREY_47                   0x78787800
#define _GRAY48                    0x7A7A7A00
#define _GRAY_48                   0x7A7A7A00
#define _GREY48                    0x7A7A7A00
#define _GREY_48                   0x7A7A7A00
#define _GRAY49                    0x7D7D7D00
#define _GRAY_49                   0x7D7D7D00
#define _GREY49                    0x7D7D7D00
#define _GREY_49                   0x7D7D7D00
#define _GRAY50                    0x7F7F7F00
#define _GRAY_50                   0x7F7F7F00
#define _GREY50                    0x7F7F7F00
#define _GREY_50                   0x7F7F7F00
#define _GRAY51                    0x82828200
#define _GRAY_51                   0x82828200
#define _GREY51                    0x82828200
#define _GREY_51                   0x82828200
#define _GRAY52                    0x85858500
#define _GRAY_52                   0x85858500
#define _GREY52                    0x85858500
#define _GREY_52                   0x85858500
#define _GRAY53                    0x87878700
#define _GRAY_53                   0x87878700
#define _GREY53                    0x87878700
#define _GREY_53                   0x87878700
#define _GRAY54                    0x8A8A8A00
#define _GRAY_54                   0x8A8A8A00
#define _GREY54                    0x8A8A8A00
#define _GREY_54                   0x8A8A8A00
#define _GRAY55                    0x8C8C8C00
#define _GRAY_55                   0x8C8C8C00
#define _GREY55                    0x8C8C8C00
#define _GREY_55                   0x8C8C8C00
#define _GRAY56                    0x8F8F8F00
#define _GRAY_56                   0x8F8F8F00
#define _GREY56                    0x8F8F8F00
#define _GREY_56                   0x8F8F8F00
#define _GRAY57                    0x91919100
#define _GRAY_57                   0x91919100
#define _GREY57                    0x91919100
#define _GREY_57                   0x91919100
#define _GRAY58                    0x94949400
#define _GRAY_58                   0x94949400
#define _GREY58                    0x94949400
#define _GREY_58                   0x94949400
#define _GRAY59                    0x96969600
#define _GRAY_59                   0x96969600
#define _GREY59                    0x96969600
#define _GREY_59                   0x96969600
#define _GRAY60                    0x99999900
#define _GRAY_60                   0x99999900
#define _GREY60                    0x99999900
#define _GREY_60                   0x99999900
#define _GRAY61                    0x9C9C9C00
#define _GRAY_61                   0x9C9C9C00
#define _GREY61                    0x9C9C9C00
#define _GREY_61                   0x9C9C9C00
#define _GRAY62                    0x9E9E9E00
#define _GRAY_62                   0x9E9E9E00
#define _GREY62                    0x9E9E9E00
#define _GREY_62                   0x9E9E9E00
#define _GRAY63                    0xA1A1A100
#define _GRAY_63                   0xA1A1A100
#define _GREY63                    0xA1A1A100
#define _GREY_63                   0xA1A1A100
#define _GRAY64                    0xA3A3A300
#define _GRAY_64                   0xA3A3A300
#define _GREY64                    0xA3A3A300
#define _GREY_64                   0xA3A3A300
#define _GRAY65                    0xA6A6A600
#define _GRAY_65                   0xA6A6A600
#define _GREY65                    0xA6A6A600
#define _GREY_65                   0xA6A6A600
#define _GRAY66                    0xA8A8A800
#define _GRAY_66                   0xA8A8A800
#define _GREY66                    0xA8A8A800
#define _GREY_66                   0xA8A8A800
#define _GRAY67                    0xABABAB00
#define _GRAY_67                   0xABABAB00
#define _GREY67                    0xABABAB00
#define _GREY_67                   0xABABAB00
#define _GRAY68                    0xADADAD00
#define _GRAY_68                   0xADADAD00
#define _GREY68                    0xADADAD00
#define _GREY_68                   0xADADAD00
#define _GRAY69                    0xB0B0B000
#define _GRAY_69                   0xB0B0B000
#define _GREY69                    0xB0B0B000
#define _GREY_69                   0xB0B0B000
#define _GRAY70                    0xB3B3B300
#define _GRAY_70                   0xB3B3B300
#define _GREY70                    0xB3B3B300
#define _GREY_70                   0xB3B3B300
#define _GRAY71                    0xB5B5B500
#define _GRAY_71                   0xB5B5B500
#define _GREY71                    0xB5B5B500
#define _GREY_71                   0xB5B5B500
#define _GRAY72                    0xB8B8B800
#define _GRAY_72                   0xB8B8B800
#define _GREY72                    0xB8B8B800
#define _GREY_72                   0xB8B8B800
#define _GRAY73                    0xBABABA00
#define _GRAY_73                   0xBABABA00
#define _GREY73                    0xBABABA00
#define _GREY_73                   0xBABABA00
#define _GRAY74                    0xBDBDBD00
#define _GRAY_74                   0xBDBDBD00
#define _GREY74                    0xBDBDBD00
#define _GREY_74                   0xBDBDBD00
#define _GRAY75                    0xBFBFBF00
#define _GRAY_75                   0xBFBFBF00
#define _GREY75                    0xBFBFBF00
#define _GREY_75                   0xBFBFBF00
#define _GRAY76                    0xC2C2C200
#define _GRAY_76                   0xC2C2C200
#define _GREY76                    0xC2C2C200
#define _GREY_76                   0xC2C2C200
#define _GRAY77                    0xC4C4C400
#define _GRAY_77                   0xC4C4C400
#define _GREY77                    0xC4C4C400
#define _GREY_77                   0xC4C4C400
#define _GRAY78                    0xC7C7C700
#define _GRAY_78                   0xC7C7C700
#define _GREY78                    0xC7C7C700
#define _GREY_78                   0xC7C7C700
#define _GRAY79                    0xC9C9C900
#define _GRAY_79                   0xC9C9C900
#define _GREY79                    0xC9C9C900
#define _GREY_79                   0xC9C9C900
#define _GRAY80                    0xCCCCCC00
#define _GRAY_80                   0xCCCCCC00
#define _GREY80                    0xCCCCCC00
#define _GREY_80                   0xCCCCCC00
#define _GRAY81                    0xCFCFCF00
#define _GRAY_81                   0xCFCFCF00
#define _GREY81                    0xCFCFCF00
#define _GREY_81                   0xCFCFCF00
#define _GRAY82                    0xD1D1D100
#define _GRAY_82                   0xD1D1D100
#define _GREY82                    0xD1D1D100
#define _GREY_82                   0xD1D1D100
#define _GRAY83                    0xD4D4D400
#define _GRAY_83                   0xD4D4D400
#define _GREY83                    0xD4D4D400
#define _GREY_83                   0xD4D4D400
#define _GRAY84                    0xD6D6D600
#define _GRAY_84                   0xD6D6D600
#define _GREY84                    0xD6D6D600
#define _GREY_84                   0xD6D6D600
#define _GRAY85                    0xD9D9D900
#define _GRAY_85                   0xD9D9D900
#define _GREY85                    0xD9D9D900
#define _GREY_85                   0xD9D9D900
#define _GRAY86                    0xDBDBDB00
#define _GRAY_86                   0xDBDBDB00
#define _GREY86                    0xDBDBDB00
#define _GREY_86                   0xDBDBDB00
#define _GRAY87                    0xDEDEDE00
#define _GRAY_87                   0xDEDEDE00
#define _GREY87                    0xDEDEDE00
#define _GREY_87                   0xDEDEDE00
#define _GRAY88                    0xE0E0E000
#define _GRAY_88                   0xE0E0E000
#define _GREY88                    0xE0E0E000
#define _GREY_88                   0xE0E0E000
#define _GRAY89                    0xE3E3E300
#define _GRAY_89                   0xE3E3E300
#define _GREY89                    0xE3E3E300
#define _GREY_89                   0xE3E3E300
#define _GRAY90                    0xE5E5E500
#define _GRAY_90                   0xE5E5E500
#define _GREY90                    0xE5E5E500
#define _GREY_90                   0xE5E5E500
#define _GRAY91                    0xE8E8E800
#define _GRAY_91                   0xE8E8E800
#define _GREY91                    0xE8E8E800
#define _GREY_91                   0xE8E8E800
#define _GRAY92                    0xEBEBEB00
#define _GRAY_92                   0xEBEBEB00
#define _GREY92                    0xEBEBEB00
#define _GREY_92                   0xEBEBEB00
#define _GRAY93                    0xEDEDED00
#define _GRAY_93                   0xEDEDED00
#define _GREY93                    0xEDEDED00
#define _GREY_93                   0xEDEDED00
#define _GRAY94                    0xF0F0F000
#define _GRAY_94                   0xF0F0F000
#define _GREY94                    0xF0F0F000
#define _GREY_94                   0xF0F0F000
#define _GRAY95                    0xF2F2F200
#define _GRAY_95                   0xF2F2F200
#define _GREY95                    0xF2F2F200
#define _GREY_95                   0xF2F2F200
#define _GRAY96                    0xF5F5F500
#define _GRAY_96                   0xF5F5F500
#define _GREY96                    0xF5F5F500
#define _GREY_96                   0xF5F5F500
#define _GRAY97                    0xF7F7F700
#define _GRAY_97                   0xF7F7F700
#define _GREY97                    0xF7F7F700
#define _GREY_97                   0xF7F7F700
#define _GRAY98                    0xFAFAFA00
#define _GRAY_98                   0xFAFAFA00
#define _GREY98                    0xFAFAFA00
#define _GREY_98                   0xFAFAFA00
#define _GRAY99                    0xFCFCFC00
#define _GRAY_99                   0xFCFCFC00
#define _GREY99                    0xFCFCFC00
#define _GREY_99                   0xFCFCFC00
#define _GRAY100                   0xFFFFFF00
#define _GRAY_100                  0xFFFFFF00
#define _GREY100                   0xFFFFFF00
#define _GREY_100                  0xFFFFFF00
#define _DARKGREY                  0xA9A9A900
#define _DARK_GREY                 0xA9A9A900
#define _DARKGRAY                  0xA9A9A900
#define _DARK_GRAY                 0xA9A9A900
#define _DARKBLUE                  0x00008B00
#define _DARK_BLUE                 0x00008B00
#define _DARKCYAN                  0x008B8B00
#define _DARK_CYAN                 0x008B8B00
#define _DARKMAGENTA               0x8B008B00
#define _DARK_MAGENTA              0x8B008B00
#define _DARKRED                   0x8B000000
#define _DARK_RED                  0x8B000000
#define _LIGHTGREEN                0x90EE9000
#define _LIGHT_GREEN               0x90EE9000

